###
### Top-level Janos Java Makefile
###
## Copyright (c) 1999, 2000, 2001, 2002 The University of Utah and the Flux Group.
### All rights reserved.
###
### This file is part of the Janos Java NodeOS software.
### 
### Permission to use, copy, modify, distribute, and sell this software and
### its documentation is hereby granted without fee, provided that the above
### copyright notice and this permission/disclaimer notice is retained in all
### copies or modified versions, and that both notices appear in supporting
### documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
### WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
### THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
### PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
### ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
### 
### Users are requested, but not required, to send to csl-dist@cs.utah.edu any
### improvements that they make and grant Univ. of Utah redistribution rights.
###

### Run 'make showhelp' if you don't want to read this file.

###
### Variables that control what is built
###
# BUILD=[debug|optimized|exposed|profiled]   [debug]
# NODEOS=[pj|moab]	                     [pj]
# OBJ_DIR=<dir>                              [`pwd`]
# SRC_DIR=<dir>                              [`pwd`]
# VERBOSEBUILD=[yes|no]                      [no]

###
### Provide defaults for command-line variables
###

ifndef BUILD
BUILD:=debug
endif

# The directory in which all output will be generated.
ifndef OBJ_DIR
OBJ_DIR:=$(shell pwd)
endif

# The top of the Janos Java NodeOS source tree
ifndef SRC_DIR
SRC_DIR:=$(shell pwd)
endif

QUIET:=@
ifeq ($(VERBOSEBUILD),yes)
QUIET:=
endif

ifndef NODEOS
NODEOS:=pj
endif

ifneq ($(NODEOS),pj)
ifneq ($(NODEOS),moab)
NODEOS:=ERROR
endif
endif

###
### "Parse" the command-line variables
###

# Default to stuff that blows up good.
LIBSTYLE:=ERROR

ifeq ($(BUILD),debug)
LIBSTYLE:=debug
endif

ifeq ($(BUILD),exposed)
LIBSTYLE:=exposed
endif

ifeq ($(BUILD),optimized)
LIBSTYLE:=optimized
endif

ifeq ($(BUILD),profiled)
LIBSTYLE:=profiled
endif

###
### Define some global values dependent upon the build
###

### Package prefix for all generated packages
PACKAGE_PREFIX:=edu.utah.janos
PACKAGE_PREFIX_DIR:=$(subst .,/,$(PACKAGE_PREFIX))

### destination directory for class files
DEST_DIR:=$(OBJ_DIR)/classes-$(NODEOS)-$(BUILD)


###
### The basic rules.  Must be defined before any includes.
###

ifeq ($(LIBSTYLE),ERROR)

# Trap bad Makefile flags here
default all: 
	@echo "FATAL ERROR in GNUmakefile Unknown BUILD flag '$(BUILD)'"
	@exit 1

else

all: allzip

install: allinstall

endif

###
### Include configuration defintions.
###

# Defines things like ECHO_CLEAN, MKDIR, etc.
-include $(SRC_DIR)/Makefiles/System.mk

# Define the PREPROCESS magic
-include $(SRC_DIR)/Makefiles/Preprocess.mk

# Define the COMPILE_JAVA command
-include $(SRC_DIR)/Makefiles/Java.mk

# Define the INSTALL_CLASS, INSTALL_ZIP commands
-include $(SRC_DIR)/Makefiles/Install.mk

# Define the JAVA_ARCHIVER command
-include $(SRC_DIR)/Makefiles/Archiver.mk

# Define the ETAGS command
-include $(SRC_DIR)/Makefiles/Tags.mk

###
### Include all of the package descriptions.  For each package
### explicitly list its dependencies.
### 
###

## Clear these variables.  They will be added to by each include of
## DefinePackage.
ALL_SOURCES:=
ALL_CPPSOURCES:=
ALL_CLASSES_GLOB:=
COMPILATION_TARGETS:=

PACKAGE_NAME:=nodeos
nodeos_PKG_DEPENDENCIES=#none
-include $(SRC_DIR)/Makefiles/DefinePackage.mk

PACKAGE_NAME:=io
io_PKG_DEPENDENCIES=#none
-include $(SRC_DIR)/Makefiles/DefinePackage.mk

PACKAGE_NAME:=route
route_PKG_DEPENDENCIES=nodeos
-include $(SRC_DIR)/Makefiles/DefinePackage.mk

PACKAGE_NAME:=util
util_PKG_DEPENDENCIES=nodeos
-include $(SRC_DIR)/Makefiles/DefinePackage.mk

PACKAGE_NAME:=protocols.ip
util_PKG_DEPENDENCIES=nodeos util
-include $(SRC_DIR)/Makefiles/DefinePackage.mk

PACKAGE_NAME:=protocols.icmp
util_PKG_DEPENDENCIES=nodeos util protocols.ip
-include $(SRC_DIR)/Makefiles/DefinePackage.mk

PACKAGE_NAME:=protocols.udp
util_PKG_DEPENDENCIES=nodeos util protocols.ip
-include $(SRC_DIR)/Makefiles/DefinePackage.mk

#PACKAGE_NAME:=services.icmp
#util_PKG_DEPENDENCIES=nodeos util protocols.icmp protocols.ip
#-include $(SRC_DIR)/Makefiles/DefinePackage.mk

PACKAGE_NAME:=services.arp
util_PKG_DEPENDENCIES=nodeos util
-include $(SRC_DIR)/Makefiles/DefinePackage.mk

#PACKAGE_NAME:=apps.ping
#util_PKG_DEPENDENCIES=nodeos util protocols.icmp protocols.ip
#-include $(SRC_DIR)/Makefiles/DefinePackage.mk

PACKAGE_NAME:=apps.passthru
util_PKG_DEPENDENCIES=nodeos util protocols.icmp protocols.ip
-include $(SRC_DIR)/Makefiles/DefinePackage.mk

PACKAGE_NAME:=benchmarks
util_PKG_DEPENDENCIES=nodeos
-include $(SRC_DIR)/Makefiles/DefinePackage.mk

PACKAGE_NAME:=tests
tests_PKG_DEPENDENCIES=nodeos util
-include $(SRC_DIR)/Makefiles/DefinePackage.mk

PACKAGE_NAME:=tests.envs
tests_PKG_DEPENDENCIES=nodeos util
-include $(SRC_DIR)/Makefiles/DefinePackage.mk

# XXX Demo is not compiled in because it relies on gross hacks (moab_Info)
#PACKAGE_NAME:=tests.demo
#tests_PKG_DEPENDENCIES=nodeos util
#-include $(SRC_DIR)/Makefiles/DefinePackage.mk

PACKAGE_NAME:=tests.nodeos
tests.nodeos_PKG_DEPENDENCIES=nodeos tests util
-include $(SRC_DIR)/Makefiles/DefinePackage.mk

###
### top-level rules
###

# A sanity check of SRC_DIR
SANITYCHECKFILE:=$(SRC_DIR)/nodeos/Main.Java
sanitycheck::
	@test -r $(SANITYCHECKFILE) \
		|| ( echo "Sanity check failed because $(SANITYCHECKFILE) is not readable."; \
			false; )

# A short blurb about what is actually being built.
statusreport:
	@echo "Building the Janos Java Libraries."
	@echo "BUILD=$(BUILD);  NODEOS=$(NODEOS);"
	@echo "SRC_DIR='$(SRC_DIR)';  DEST_DIR='$(DEST_DIR)'"


allpackages: $(COMPILATION_TARGETS)
	@echo "All packages are up to date."

JNODEOS_ARCHIVE:=jnodeos-$(NODEOS)-$(BUILD).$($(JAVA_ARCHIVER_TAG)_EXTENSION)
JNODEOSTESTS_ARCHIVE:=jnodeostests-$(NODEOS)-$(BUILD).$($(JAVA_ARCHIVER_TAG)_EXTENSION)

.PHONY: jnodeos.zip jnodeos.jar
jnodeos.zip jnodeos.jar: $(DEST_DIR)/$(JNODEOS_ARCHIVE)
jnodeostests.zip jnodeostests.jar: $(DEST_DIR)/$(JNODEOSTESTS_ARCHIVE)

ifeq ($(JAVA_ARCHIVER_TAG),none)
allzip: $(DEST_DIR)/$(JNODEOS_ARCHIVE) $(DEST_DIR)/$(JNODEOSTESTS_ARCHIVE)
	@echo "Completed build."

$(DEST_DIR)/$(JNODEOS_ARCHIVE): $(COMPILATION_TARGETS)
	@echo "Skipping construction of jnodeos archive; no archiver configured."

$(DEST_DIR)/$(JNODEOSTESTS_ARCHIVE): $(COMPILATION_TARGETS)
	@echo "Skipping construction of jnodeos tests archive; no archiver configured."
else
allzip: $(DEST_DIR)/$(JNODEOS_ARCHIVE) $(DEST_DIR)/$(JNODEOSTESTS_ARCHIVE)
	@echo "All archives are up to date."

ARCHIVER:=$(JAVA_ARCHIVER) $($(JAVA_ARCHIVER_TAG)_OPTS)
$(DEST_DIR)/$(JNODEOS_ARCHIVE): $(COMPILATION_TARGETS)
	@echo "Updating $(JNODEOS_ARCHIVE)..."
# XXXX this is getting gross
	$(QUIET) (cd $(DEST_DIR); \
		$(ARCHIVER) $@ \
			$(nodeos_CLASSES_GLOB) $(io_CLASSES_GLOB) \
			$(route_CLASSES_GLOB) $(util_CLASSES_GLOB) \
			$(protocols.ip_CLASSES_GLOB) \
			$(protocols.udp_CLASSES_GLOB) \
			$(protocols.icmp_CLASSES_GLOB) \
			$(services.icmp_CLASSES_GLOB) \
			$(services.arp_CLASSES_GLOB) \
			$(apps.ping_CLASSES_GLOB) \
			$(apps.passthru_CLASSES_GLOB) \
			$(benchmarks_CLASSES_GLOB) \
		)
	@echo "... done."

$(DEST_DIR)/$(JNODEOSTESTS_ARCHIVE): $(COMPILATION_TARGETS)
	@echo "Updating $(JNODEOSTESTS_ARCHIVE)..."
	$(QUIET) (cd $(DEST_DIR); \
		$(ARCHIVER) $@ \
			$(tests_CLASSES_GLOB) \
			$(tests.nodeos_CLASSES_GLOB) \
			$(tests.demo_CLASSES_GLOB) \
			$(tests.envs_CLASSES_GLOB) \
		)
	@echo "... done."
endif

allinstall: $(DEST_DIR)/$(JNODEOS_ARCHIVE)
	test -d $(INSTALL_DIR)               || mkdir $(INSTALL_DIR) 
	test -d $(INSTALL_DIR)/share         || mkdir $(INSTALL_DIR)/share
	test -d $(INSTALL_DIR)/share/jnodeos || mkdir $(INSTALL_DIR)/share/jnodeos
	@echo "Installing $(JNODEOS_ARCHIVE) in $(INSTALL_DIR)/$(JNODEOS_ARCHIVE)"
	$(QUIET) $(INSTALL_ZIP) $(DEST_DIR)/$(JNODEOS_ARCHIVE) $(INSTALL_DIR)/share/jnodeos/$(JNODEOS_ARCHIVE)

### Some "online" documentation.
showhelp:
	@echo "Janos Java Bits Construction Kit:"
	@echo "  make [BUILD=<debug|optimized|exposed|profiled>] [NODEOS=<pj|moab>] [all|debug|clean]"
	@echo " "
	@echo "  Variables:"
	@echo "    BUILD controls the type of .class files built (optimized, debugging, etc)."
	@echo "      (exposed means all fields and methods are public.  Can be handy.)"
	@echo "      (profiled means xprofiling stages will be created.)"
	@echo "    NODEOS controls the underyling Node OS that is targetted."
	@echo " "
	@echo "  Targets:"
	@echo "    The 'all' target (default) builds all the classes."
	@echo "    The 'install' target installs all the archives (And builds them if necessary)."
	@echo "    The 'mkdebug' target prints gobs of debugging info *about the makefiles*"
	@echo "       (try: 'mkdebug_Java.mk', 'mkdebug_System.mk', 'mkdebug_nodeos', etc.)."
	@echo "    The 'clean' target removes all of the generated .java and .class files."
	@echo "    The 'showhelp' target is not documented."


# All the .mk files should be available.  Otherwise, something is broken
%.mk:
	@echo "FATAL ERROR: Required makefile fragment $@ is not available."
	@exit 1

### Generic clean target (nothing to do at this level)
clean::
	@echo "Removing $(JNODEOS_ARCHIVE)"
	$(QUIET)$(RM) -f $(DEST_DIR)/$(JNODEOS_ARCHIVE)

### Generic tags target (nothing to do at this level).  See DefinePackage.mk
tags:
	@rm -f TAGS
	$(QUIET) echo "Combining TAGS files for each package"
	$(QUIET) cat TAGS_* > TAGS
	@rm -f TAGS_*

distclean::
	$(RM) $(OBJ_DIR)/.compile-*
	@for nodeos in pj moab; do		\
		for way in debug optimized exposed profiled; do \
			echo "rm -rf $(OBJ_DIR)/classes-$$nodeos-$$way";  \
			rm -rf $(OBJ_DIR)/classes-$$nodeos-$$way; \
			echo "rm -f $(DEST_DIR)/{classes,jndoeos}-$$nodeos-$$way.{zip,jar}"; \
			rm -f $(DEST_DIR)/classes-$$nodeos-$$way.{zip,jar}; \
		done;				\
	done
	@for file in $(DISTCLEAN_FILES); do \
		echo "$(RM) $$file"; \
		$(RM) $$file; \
	done

### Generic mkdebug target (all included makefiles have a mkdebug::
### target)
mkdebug::  mkdebug_Main.mk

mkdebug_Main.mk::
	@echo 
	@echo "**** Main.mk info:"
	@echo "    ALL_SOURCES = $(ALL_SOURCES)"
	@echo "    ALL_CPPSOURCES = $(ALL_CPPSOURCES)"
	@echo "    COMPILATION_TARGETS = $(COMPILATION_TARGETS)"
	@echo "    INSTALL_DIR         = $(OBJ_DIR)"
	@echo "    SRC_DIR             = $(SRC_DIR)"
	@echo "    OBJ_DIR             = $(OBJ_DIR)"
	@echo "    DEST_DIR            = $(DEST_DIR)"
	@echo "    PACKAGE_PREFIX_DIR  = $(PACKAGE_PREFIX_DIR)"
	@echo "    QUIET               = $(QUIET)"


# Most of this makefile is phony.
.PHONY: default all install
.PHONY: sanitycheck statusreport showhelp
.PHONY: allpackages allzip allinstall
.PHONY: mkdebug mkdebug_Main.mk 

# eof
