/*
 * PassThru.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_APPS_PKG.passthru;

import JANOS_NODEOS_PKG.Domain;
import JANOS_NODEOS_PKG.Node;
import JANOS_NODEOS_PKG.DomainRun;
import JANOS_NODEOS_PKG.Address;
import JANOS_NODEOS_PKG.TimeSpec;
import JANOS_NODEOS_PKG.ChanStats;
import JANOS_NODEOS_PKG.InChannel;
import JANOS_NODEOS_PKG.OutChannel;
import JANOS_NODEOS_PKG.DeliverFunc;
import JANOS_NODEOS_PKG.BufferHandle;
import JANOS_NODEOS_PKG.DemultiplexKey;
import JVM_COMMSPACE_PKG.CommSpaceHandle;

import java.util.Vector;

public class PassThru
	extends DomainRun
{
	private class Link
		implements DeliverFunc
	{
		private String linkName;
		private InChannel ic;
		private OutChannel oc;

		public Link(String linkName,
			    String inProtocol,
			    String inAddress,
			    String outProtocol,
			    String outAddress)
			    throws Exception
		{
			DemultiplexKey dk = new DemultiplexKey(PassThru.this.
							       domain);

			this.linkName = linkName;
			this.ic = InChannel.create(PassThru.this.domain,
						   dk,
						   inProtocol,
						   inAddress,
						   this);
			this.ic.addBuffers(32);
			this.oc = OutChannel.create(PassThru.this.domain,
						    outProtocol,
						    outAddress);
			Node.println("Link: "
				     + inProtocol
				     + "/"
				     + inAddress
				     + " -> "
				     + outProtocol
				     + "/"
				     + outAddress);
		}

		public void receive(BufferHandle bh, int payloadOffset)
		{
			this.oc.send(bh);
			this.ic.addBuffer(bh);
		}

		public void printStats()
		{
			ChanStats cs = new ChanStats();

			Node.println("Stats for link "
				     + this.linkName
				     + ":");
			this.ic.getStats(cs);
			Node.println("  InChannel:  "
				     + cs);
			this.oc.getStats(cs);
			Node.println("  OutChannel:  "
				     + cs);
		}
	}
	
	private Domain domain;
	private Vector links = new Vector();
	private int RUNFOR = 30; // Seconds

	protected void configure()
		throws Exception
	{
		String outProtocol, outAddress;
		String inProtocol, inAddress;
		String link;
		int lpc;
		
		for( lpc = 0;
		     (link = Domain.lookupConfig("PassThru:links", "link", lpc))
			     != null;
		     lpc++ )
		{
			outProtocol = Domain.lookupConfig("PassThru:" + link,
							"output-protocol");
			if( outProtocol == null )
				throw new Exception("No output-protocol "
						    + "specified for link: "
						    + link);
			outAddress = Domain.lookupConfig("PassThru:" + link,
						       "output-address");
			if( outAddress == null )
				throw new Exception("No output-address "
						    + "specified for link: "
						    + link);
			inProtocol = Domain.lookupConfig("PassThru:" + link,
						       "input-protocol");
			if( inProtocol == null )
				throw new Exception("No input-protocol "
						    + "specified for link: "
						    + link);
			inAddress = Domain.lookupConfig("PassThru:" + link,
						      "input-address");
			if( inAddress == null )
				throw new Exception("No input-address "
						    + "specified for link: "
						    + link);
			this.links.addElement(new Link(link,
						inProtocol,
						inAddress,
						outProtocol,
						outAddress));
		}
		if( lpc == 0 )
		{
			throw new Exception("No links specified in config");
		}
	}

	protected void run(Domain domain,
			   String args[],
			   CommSpaceHandle csh)
	{
		this.domain = domain;
		try
		{
			String str;
			
			this.configure();
			str = Domain.lookupConfig("PassThru", "runfor");
			if( str != null )
				RUNFOR = Integer.parseInt(str);
			if( RUNFOR > 0 )
			{
				int lpc;
				
				Thread.sleep(RUNFOR * 1000);
				for( lpc = 0; lpc < this.links.size(); lpc++ )
				{
					((Link)this.links.elementAt(lpc)).
						printStats();
				}
				Node.shutdown();
			}
		}
		catch(Exception e)
		{
			Node.println("Unable to start domain: " + e);
			Node.printStackTrace(e);
		}
	}
	
}
