/*
 * PingArgs.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_APPS_PKG.ping;

import JANOS_NODEOS_PKG.Domain;
import JANOS_NODEOS_PKG.Address;
import JANOS_NODEOS_PKG.AddressFormatException;

import JANOS_PROTOCOLS_PKG.ip.IPHeader;

/**
 * Simple class to hold any arguments to the ping domain.
 */
PACKAGE class PingArgs
{
	private boolean audible = false;
	private int count = -1;
	private boolean flood = false;
	private int waitTime = 1;
	private int source;
	private int destination;
	private byte ttl = IPHeader.MAX_TTL;
	private String inputProtocolSpec;
	private String inputAddressSpec;
	private String outputProtocolSpec;
	private String outputAddressSpec;

	PACKAGE PingArgs(String args[])
		throws NumberFormatException, AddressFormatException
	{
		int lpc;

		/*
		 * We pull this stuff out of the config file so we don't have
		 * to type it in on the command line every time.
		 */
		this.source = Address.fromString(
			Domain.lookupConfig("BasicConfig", "address"));
		this.inputProtocolSpec = Domain.
			lookupConfig("BasicConfig:input0", "protocol");
		this.inputAddressSpec = Domain.
			lookupConfig("BasicConfig:input0", "address");
		this.outputProtocolSpec = Domain.
			lookupConfig("BasicConfig:output0", "protocol");
		this.outputAddressSpec = Domain.
			lookupConfig("BasicConfig:output0", "address");
		for( lpc = 0; lpc < args.length; lpc++ )
		{
			if( args[lpc].equals("-a") )
			{
				this.audible = true;
			}
			else if( args[lpc].equals("-c") )
			{
				this.count = Integer.parseInt(args[lpc + 1]);
				lpc++;
			}
			else if( args[lpc].equals("-f") )
			{
				this.flood = true;
			}
			else if( args[lpc].equals("-i") )
			{
				this.waitTime =
					Integer.parseInt(args[lpc + 1]);
				lpc++;
			}
			else if( args[lpc].equals("-T") )
			{
				this.ttl = Byte.parseByte(args[lpc + 1]);
				lpc++;
			}
			else
			{
				this.destination =
					Address.fromString(args[lpc]);
			}
		}
	}
	
	PACKAGE boolean getAudible()
	{
		return this.audible;
	}

	PACKAGE int getCount()
	{
		return this.count;
	}

	PACKAGE boolean getFlood()
	{
		return this.flood;
	}

	PACKAGE int getWaitTime()
	{
		return this.waitTime;
	}

	PACKAGE int getSource()
	{
		return this.source;
	}

	PACKAGE int getDestination()
	{
		return this.destination;
	}

	PACKAGE byte getTTL()
	{
		return this.ttl;
	}

	PACKAGE String getInputProtocolSpec()
	{
		return this.inputProtocolSpec;
	}
	
	PACKAGE String getInputAddressSpec()
	{
		return this.inputAddressSpec;
	}
	
	PACKAGE String getOutputProtocolSpec()
	{
		return this.outputProtocolSpec;
	}
	
	PACKAGE String getOutputAddressSpec()
	{
		return this.outputAddressSpec;
	}
}
