/*
 * Flood.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_BENCHMARKS_PKG;

import java.io.IOException;

import JANOS_NODEOS_PKG.Domain;
import JANOS_NODEOS_PKG.Node;
import JANOS_NODEOS_PKG.DomainRun;
import JANOS_NODEOS_PKG.OutChannel;
import JANOS_NODEOS_PKG.BufferHandle;
import JVM_COMMSPACE_PKG.CommSpaceHandle;
import JANOS_NODEOS_PKG.ProtocolSpecFormatException;
import JANOS_NODEOS_PKG.PacketAddressFormatException;

public class Flood
	extends DomainRun
{
	private Domain domain;

	public static final byte headerByte = (byte)0xf1;
	private static final int packetLength = 2;

	private OutChannel createOutChannel(String args[])
		throws ProtocolSpecFormatException,
		       PacketAddressFormatException,
		       IOException
	{
		String protocolSpec, addressSpec;
		OutChannel retval;

		if( args.length >= 2 )
		{
			protocolSpec = args[0];
			addressSpec = args[1];
		}
		else
		{
			protocolSpec = Domain.lookupConfig("FloodConfig",
							 "protocol");
			addressSpec = Domain.lookupConfig("FloodConfig",
							"address");
		}
		retval = OutChannel.create(this.domain,
					   protocolSpec,
					   addressSpec);
		return retval;
	}

	private int iterations(String args[])
		throws NumberFormatException
	{
		String str = "50000";
		int retval;

		if( args.length == 3 )
		{
			str = args[2];
		}
		else
		{
			str = Domain.lookupConfig("FloodConfig", "iterations");
		}
		retval = Integer.parseInt(str);
		return retval;
	}
	
	private BufferHandle createPacket(byte val)
	{
		BufferHandle retval;

		retval = BufferHandle.create(0, packetLength);
		retval.setByte(0, headerByte);
		retval.setByte(1, val);
		return retval;
	}

	protected void run(Domain domain,
			   String args[],
			   CommSpaceHandle csh)
	{
		try
		{
			BufferHandle bh;
			OutChannel oc;
			int lpc, iter;
			
			this.domain = domain;
			oc = this.createOutChannel(args);
			bh = this.createPacket((byte)0);
			iter = this.iterations(args);
			for( lpc = 0; lpc < iter; lpc++ )
			{
				oc.send(bh);
				if( (lpc % 1000) == 0 )
				{
					Node.println("Sent "
						     + lpc
						     + " packets...");
				}
			}
		}
		catch(Exception e)
		{
			Node.println("Exception " + e);
			Node.printStackTrace(e);
		}
	}
}
