/*
 * FloodSink.Java
 *
 * Copyright (c) 2000, 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_BENCHMARKS_PKG;

import java.io.IOException;

import JANOS_NODEOS_PKG.Domain;
import JANOS_NODEOS_PKG.Node;
import JANOS_NODEOS_PKG.DomainRun;
import JANOS_NODEOS_PKG.TimeSpec;
import JANOS_NODEOS_PKG.InChannel;
import JANOS_NODEOS_PKG.DeliverFunc;
import JANOS_NODEOS_PKG.BufferHandle;
import JANOS_NODEOS_PKG.DemultiplexKey;
import JVM_COMMSPACE_PKG.CommSpaceHandle;
import JANOS_NODEOS_PKG.ProtocolSpecFormatException;
import JANOS_NODEOS_PKG.PacketAddressFormatException;

public class FloodSink
	extends DomainRun
	implements DeliverFunc
{
	private Domain domain;
	private InChannel inChannel;
	
	private TimeSpec startTime = new TimeSpec();
	private TimeSpec endTime = new TimeSpec();
	
	private int totalReceived = 0;
	private int expectedPackets = 8000;

	private InChannel createInChannel(String args[])
		throws ProtocolSpecFormatException,
		       PacketAddressFormatException,
		       IOException
	{
		String protocolSpec, addressSpec;
		DemultiplexKey dk;
		InChannel retval;

		if( args.length >= 2 )
		{
			protocolSpec = args[0];
			addressSpec = args[1];
		}
		else
		{
			protocolSpec = Domain.lookupConfig("FloodConfig",
							 "protocol");
			addressSpec = Domain.lookupConfig("FloodConfig",
							"address");
		}
		dk = new DemultiplexKey(this.domain);
		dk.add(Flood.headerByte);
		retval = InChannel.create(this.domain,
					  dk,
					  protocolSpec,
					  addressSpec,
					  this);
		retval.addBuffers(10);
		return retval;
	}

	public void receive(BufferHandle bh, int payloadOffset)
	{
		this.totalReceived++;
		if( (this.totalReceived % 100) == 0 )
		{
			Node.println("Sent "
				     + this.totalReceived
				     + " packets...");
		}
		if( this.totalReceived == 1 )
		{
			TimeSpec.getTime(this.startTime);
		}
		else if( this.totalReceived == this.expectedPackets )
		{
			double diff, seconds;
			
			TimeSpec.getTime(this.endTime);
			Node.println("StartTime = "
				     + this.startTime
				     + "; EndTime = "
				     + this.endTime);
			this.endTime.subtract(this.startTime);
			diff = (double)this.endTime.getTotalMillis();
			seconds = diff / (double)1000.0;
			Node.println("Time = "
				     + seconds
				     + "s; pps = "
				     + ((double)this.totalReceived / seconds));
		}
		this.inChannel.addBuffer(bh);
	}
	
	protected void run(Domain domain,
			   String args[],
			   CommSpaceHandle csh)
	{
		try
		{
			this.domain = domain;
			this.inChannel = this.createInChannel(args);
		}
		catch(Exception e)
		{
			Node.println("Exception " + e);
			Node.printStackTrace(e);
		}
	}
}
