/*
 * AddressFactory.Java
 *
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_NODEOS_PKG;

/**
 * The Factory for creating Address objects.  Only one
 * Address factory is created, and it is created by
 * a static initalizer in Address.
 *
 * @see Address
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
public class AddressFactory
{
	/**
	 * Package-private constructor.  Only invoked in Address.
	 *
	 * @see Address.getAddressFactory
	 */
	public AddressFactory()
	{
	}

	/**
	 * Construct a new Address object that associates
	 * the given, virtual "active address" with the
	 * physical address described by the addressSpec
	 * and its protocolSpec.
	 *
	 * @param anAddress the 32-bit IPv4-like active address.
	 * @param protocolSpec the protocol context for the addressSpec
	 * @param addressSpec the address specification
	 * @return a new Address object
	 * @allocation a single Address object
	 * @see InChannel#create(Domain,ThreadPoolSpec,DemultiplexKey,String,String,DeliverFunc)
	 * @see Address
	 */
	public Address newAddress(int anAddress,
				  String protocolSpec,
				  String addressSpec)
		throws NoSuchNodeAddress
	{
		return new Address(anAddress, protocolSpec, addressSpec);
	}
}
