/*
 * CycleTimer.Java
 *
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_NODEOS_PKG;

#undef HAVE_NATIVE_CYCLETIMER // XXX

/**
 * Access to the underlying cycle timer.
 * <p>
 * XXX Make this more generic: a simple timer interface.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
public class CycleTimer
{
	public String name;
	public long count;
	public long time;
	public long timeSquareSum;

	public long start_time;

	public CycleTimer()
	{
	}

	public CycleTimer(String name)
	{
		this.name = name;
	}

	public synchronized void reset()
	{
		start_time = 0;
		count = 0;
		time = 0;
		timeSquareSum = 0;
	}

	public synchronized void start()
	{
		start_time = cycles();
	}

	public synchronized void end()
	{
		if(start_time != 0)
		{
			long dt = cycles() - start_time;
			time += dt;
			timeSquareSum += dt * dt;
			count++;
		}
	}

	public synchronized long avg()
	{
		return time / count;
	}

	public synchronized long stdDev()
	{
		double avg = time / count;
		return (long) Math.sqrt(
			((double) timeSquareSum) / count - avg * avg);
	}

#ifdef HAVE_NATIVE_CYCLETIMER
	public static native long cycles();

	static
	{
		System.loadLibrary("CycleTimer");
	}
#else
	public static long cycles() {return 0;}
#endif

}
