/*
 * DomainHandle.Java
 *
 * Copyright (c) 1999, 2000, 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_NODEOS_PKG;

/**
 * Parent's handle on a child domain.  Returned from Domain.newDomain().
 * <p>
 * The DomainHandle is many-to-one and fixed with a child Domain.  The
 * particulars of this "cross-domain pointer" are implemented in the
 * native DomainHandle.
 *
 * @see Domain
 * @see Domain#newDomain(Credentials,ThreadPoolSpec,MemSpec,DomainStartSpec,Runnable)
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
public final class DomainHandle
	extends NATIVE_DOMAIN_HANDLE
{
	PACKAGE DomainHandle(Domain f)
	{
		super(f);
	}

	/**
	 * @param fs An object to hold the domain statistics.
	 */
	public void getStats(DomainStats fs)
	{
		super.getStats(fs.getNative());
	}

	/**
	 * Get the statistics for an in channel in the domain.  InChannels don't
	 * really have a `name' attached to them so we address them with
	 * instance numbers.  Typically, if a parent domain was interested in
	 * what the children were doing it would increment the instance number
	 * until the function returned false, indicating that there weren't
	 * anymore channels.
	 *
	 * @param index An in channel instance to get stats from.
	 * @param cs The object that will hold the channels stats.
	 * @return True if the channel existed, false otherwise.
	 */
	public boolean getInChannelStats(int index, ChanStats cs)
	{
		return super.getInChannelStats(index, cs.getNative());
	}

	/**
	 * Similar to the above except for out channels.
	 *
	 * @param index An out channel instance to get stats from.
	 * @param cs The object that will hold the channels stats.
	 * @return True if the channel existed, false otherwise.
	 */
	public boolean getOutChannelStats(int index, ChanStats cs)
	{
		return super.getOutChannelStats(index, cs.getNative());
	}
	

	public String toString()
	{
		return super.toString();
	}

	PACKAGE NATIVE_DOMAIN_HANDLE getNativeDomainHandle()
	{
		return this;
	}

	/**
	 * Destroy the Domain.  The termination handler in the
	 * parent domain will be invoked.
	 */
	public void destroy()
	{
		// Override it to make it public (package-access in superclass).
		getNativeDomainHandle().nativeDestroy();
	}

	/**
	 * Get the string name of the domain pointed to by this handle.
	 */
	PACKAGE String getName()
	{
#ifdef DEBUG
		return getNativeDomainHandle().getName();
#else
		return null;
#endif
	}
}
