/*
 * DomainLocal.Java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_NODEOS_PKG;

import java.util.Hashtable;

/**
 * A ThreadLocal like class that stores data local to the domain.
 */
public class DomainLocal
{
	/**
	 * @return The initial value of the DomainLocal.
	 */
	protected Object initialValue()
	{
		return null;
	}

	/**
	 * @return The current value of the DomainLocal.
	 */
	public Object get()
	{
		return get(Domain.currentDomain());
	}

	/**
	 * @param value The value to put in the DomainLocal.
	 */
	public void set(Object value)
	{
		set(Domain.currentDomain(), value);
	}

	/**
	 * @param domain The domain to use as context when retrieving the local.
	 * @return The current value of the DomainLocal.
	 */
	private Object get(Domain domain)
	{
		Hashtable ht = domain.getLocals();
		Object retval;
	
		synchronized(ht)
		{
			if( !ht.containsKey(this) )
			{
				set(domain, this.initialValue());
			}
			retval = ht.get(this);
		}
		return retval;
	}

	/**
	 * @param domain The domain to use as context when setting the local.
	 * @param value The value to put in the local.
	 */
	private void set(Domain domain, Object value)
	{
		Hashtable ht = domain.getLocals();

		synchronized(ht)
		{
			ht.put(this, value);
		}
	}

	public String toString()
	{
		return this.getClass().getName() + "|Context[value="
			+ this.get()
			+ "]";
	}
}
