/*
 * DomainRun.Java
 *
 * Copyright (c) 1999, 2000, 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_NODEOS_PKG;

import JVM_COMMSPACE_PKG.CommSpaceHandle;

/**
 * A protocol should override <code>DomainRun</code> so that a domain can
 * be created for it.  When the domain is created, the overridden
 * <code>run()</code> method is called.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 * @see DomainStartSpec
 * @see Domain#newDomain(Credentials,ThreadPoolSpec,MemSpec,DomainStartSpec,Runnable)
 */
public abstract class DomainRun
{
	/**
	 * Invoked as the initial entrypoint in a new domain.  The domain
	 * parameter is a parameter is the new domain object, the args
	 * array contains "command line" arguments to the new domain.
	 * The CommSpaceHandle provides a communication link with the
	 * parent domain (if the parent domain provided one).
	 *
	 * @param domain the new domain and the current domain
	 * @param args whatever "command line" arguments the creator passed.
	 * @param csh handle on a parent's CommSpace
	 */
	protected abstract void run(Domain domain,
				    String[] args,
				    CommSpaceHandle csh);
}
