/*
 * ThreadPoolSpec.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_NODEOS_PKG;

/**
 * ThreadPoolSpec wrapper for Java.
 * <p>
 * The Java ThreadPoolSpec only presents the implicit thread interface
 * to Java code.  Internally, it uses explicit threads for internal
 * tracking and management.
 *
 * @see Domain
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
public final class ThreadPoolSpec
	extends NATIVE_THREADPOOLSPEC
{
	/**
	 * @param maxThreads The maximum number of threads in the pool.
	 * @param stackSize The stack size for the threads.
	 * @param scheduler The scheduler object.
	 */
	public ThreadPoolSpec(int maxThreads,
			      int stackSize,
			      ThreadScheduler scheduler)
	{
		super(maxThreads, stackSize, ThreadPoolSpec.getMaxRate() / 100,
		      scheduler == null ? ThreadScheduler.basicScheduler() : scheduler);
	}

	/**
	 * @param maxThreads The maximum number of threads in the pool.
	 * @param stackSize The stack size for the threads.
	 * @param rate The CPU rate allocation for the threads.
	 * @param scheduler The scheduler object.
	 */
	public ThreadPoolSpec(int maxThreads,
			      int stackSize,
			      int rate,
			      ThreadScheduler scheduler)
	{
		super(maxThreads, stackSize, rate,
		      scheduler == null ? ThreadScheduler.basicScheduler() :
		      scheduler);
	}

	/**
	 * @return The native ThreadPoolSpec.
	 */
	PACKAGE NATIVE_THREADPOOLSPEC getNativeSpec()
	{
		return this;
	}

	/**
	 * @return A reasonable default ThreadPoolSpec.
	 */
	public static ThreadPoolSpec getDefault()
	{
		/* XXX Come up with some real values here... */
		return new ThreadPoolSpec(2, 16384, null);
	}

	/**
	 * @return The maximum CPU rate for a thread pool.
	 */
	public static int getMaxRate()
	{
		return NATIVE_THREADPOOLSPEC.getMaxRate();
	}
}
