/*
 * moab_DemultiplexKey.Java
 *
 * Copyright (c) 2000, 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_NODEOS_PKG;

/**
 * A key for matching incoming packets.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
PACKAGE class moab_DemultiplexKey
{
	CHECK_NATIVE_NODEOS

	private native void init(moab_Domain domain);

	PACKAGE moab_DemultiplexKey(moab_Domain domain)
	{
		require(domain);
		require(domain == moab_Domain.native_currentDomain());
		init(domain);
	}

	PACKAGE native void add(int length,
				  byte sequence[], byte mask[]);

	PACKAGE native void addByte(byte b);

	PACKAGE native void addShort(short s);

	PACKAGE native void addInt(int i);

	PACKAGE native void addLong(long l);

	PACKAGE native void addETH(int flags,
				   byte src[], byte dst[], short type);
	
	PACKAGE native void addIPv4(int flags,
				    int source, int srcMask,
				    int dest, int dstMask,
				    byte protocol);

	PACKAGE native void addUDP(int srcPort, int destPort);

	PACKAGE native void addTCP(int srcPort, int destPort);
	
	PACKAGE native void addANEP(byte version,
				      boolean allVersions, short protocolID);

	PACKAGE native int matches(byte payload[],
				   int payloadStart,
				   int payloadLength);

	PACKAGE native int length();
	
	PACKAGE native int toString(char[] outBuf);

	public String toString()
	{
		char[] buf = new char[512];
		int charsUsed;
		charsUsed = toString(buf);

		StringBuffer retval = new StringBuffer(256);
		retval.append("moab_DemultiplexKey[")
			.append(buf, 0, charsUsed)
			.append("]");
		return retval.toString();
	}
}
