/*
 * moab_DomainHandle.Java
 *
 * Copyright (c) 2000, 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_NODEOS_PKG;

import JVM_SYS_PKG.Team;
import JVM_SYS_PKG.Importable;
import JVM_SYS_PKG.Exportable;
import JVM_SYS_PKG.TeamReservation;
import JVM_SYS_PKG.DeadTeamException;
import JVM_SYS_PKG.TeamEjectionException;

/**
 * See the implementation-independent DomainHandle for a description.
 *
 * @see DomainHandle
 */
PACKAGE class moab_DomainHandle
	extends Importable
{
	private Domain domain;
	
	/**
	 * Create a new domain handle that points at the
	 * given domain.
	 */
	PACKAGE moab_DomainHandle(Domain f)
	{
	}

	protected void bindTo(Exportable ex)
	{
		this.domain = (Domain)ex;
	}

	protected void unbindFrom()
	{
		this.domain = null;
	}

	PACKAGE synchronized void getStats(moab_DomainStats fs)
	{
		this.domain.getStats(fs);
	}

	PACKAGE synchronized boolean getInChannelStats(int index,
						       moab_ChanStats cs)
	{
		boolean retval = false;

		if( this.domain != null )
		{
			moab_Domain mf = this.domain.getNativeDomain();

			try
			{
				InChannel ic;
				
				ic = mf.getInChannel(index);
				ic.getStats(cs);
				retval = true;
			}
			catch(ArrayIndexOutOfBoundsException e)
			{
			}
		}
		return retval;
	}

	PACKAGE synchronized boolean getOutChannelStats(int index,
							moab_ChanStats cs)
	{
		boolean retval = false;

		if( this.domain != null )
		{
			moab_Domain mf = this.domain.getNativeDomain();

			try
			{
				OutChannel oc;
				
				oc = mf.getOutChannel(index);
				oc.getStats(cs);
				retval = true;
			}
			catch(ArrayIndexOutOfBoundsException e)
			{
			}
		}
		return retval;
	}

	PACKAGE void nativeDestroy()
	{
		require(this.domain);
		require(Team.current());

		synchronized(this)
		{
			/*
			 * XXX have to use a reservation because
			 * this.domain() gets nulled out during the Team
			 * termination.
			 */
			try
			{
				TeamReservation tr;

				tr = this.domain.pushOwnerReservation();
				tr.visit();
				this.domain.destroy();
				tr.leave();
				tr.pop();
			}
			catch (TeamEjectionException tee)
			{
				Node.println("Shouldn't be here TEE in nativeDestroy()");
			}
			catch (DeadTeamException dte)
			{
				Node.println("Shouldn't be here DTE in nativeDestroy()");
			}
		}
	}

#ifdef DEBUG
	PACKAGE synchronized String getName()
	{
		String n;
		
		synchronized(this)
		{
			// Create a copy of the name in this team
			n = new String(this.domain.getName());
		}

		return n;
	}
#endif
}
