/*
 * moab_InChannel.Java
 *
 * Copyright (c) 1999, 2000, 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_NODEOS_PKG;

import java.io.IOException;

import JVM_SYS_PKG.Team;
import JVM_SYS_PKG.Exportable;
import JVM_SYS_PKG.TemporaryExport;
import JVM_SYS_PKG.NoSuchExportException;

/**
 * See pj_InChannel for doc.
 *
 * @see pj_InChannel
 * @see InChannel
 * @see OutChannel
 * @see moab_OutChannel
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
PACKAGE class moab_InChannel
	extends Exportable
	implements TemporaryExport
{
	CHECK_NATIVE_NODEOS

	PACKAGE moab_InChannel(moab_Domain domain,
			       int threadCount,
			       moab_DemultiplexKey classifier,
			       String protocolSpec,
			       String addressSpec,
			       DeliverFunc deliverFunc)
		throws IOException,
		PacketAddressFormatException,
		ProtocolSpecFormatException
	{
		moab_ChannelAddressSpec cas;
		moab_TransportInterface ti;
		String topProtocol;
		
		// XXX the "socket:" thing is a hack for Moab
		if (protocolSpec.equals("if/ipv4/udp"))
		{
			protocolSpec = "socket:" + protocolSpec;
		}
		/*
		 * Get the top protocol that the address spec should
		 * correspond too.
		 */
		topProtocol = ProtocolSpec.lastComponent(protocolSpec);
		if( (ti = moab_Info.getTransports().
		     findInterface(topProtocol)) != null )
		{
			/*
			 * Found the interface for the top protocol, now create
			 * and initialize an address spec that moab can use.
			 */
			cas = (moab_ChannelAddressSpec)
				ti.createAddress(addressSpec);
		}
		else
		{
			String msg =
				"Protocol spec '"
				+ protocolSpec
				+ "' is not supported";
			
			throw new ProtocolSpecFormatException(msg);
		}

		/* Initialize the native data. */
		int rc = init(domain,
			      threadCount,
			      classifier,
			      protocolSpec,
			      cas,
			      deliverFunc);

		if (rc != 0)
		{
			throw new IOException("NodeOS failed to create inchannel.");
		}
		else
		{
			domain.addChannel(this);
			Team.current().exportAnonymousObject(this);
		}
	}
	
	private native int init(moab_Domain domain,
				int threadCount,
				moab_DemultiplexKey classifier,
				String protocolSpec,
				moab_ChannelAddressSpec cas,
				DeliverFunc deliverFunc);

	PACKAGE native void native_addBuffer(moab_BufferHandle bufh);
	
	PACKAGE native BufferHandle native_remBuffer();

	PACKAGE native void getStats(moab_ChanStats cs);
	
	PACKAGE native boolean handoff(moab_BufferHandle pkt);

	public String toString()
	{
		return "moab_InChannel[...]";
	}

	private native void native_destroy();

	public boolean expired()
	{
		return true;
	}
	
	PACKAGE void destroy()
	{
		try
		{
			Team.current().revokeAnonymousObject(this);
		}
		catch(NoSuchExportException e)
		{
		}
	}
	
	protected void finalize()
	{
		// XXX ??
	}
}
