/*
 * moab_Info.Java
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_NODEOS_PKG;

import java.util.Vector;

/**
 * An interface that represents a host address.  The implementation
 * is tied to the protocol layer the address is associated with.
 */
PACKAGE class moab_Info
{
	/**
	 * The available packet transports.
	 */
	private static moab_Transports transports;

	/**
	 * @return The transports object for this node as a whole.
	 */
	PACKAGE static moab_Transports getTransports()
	{
		return transports;
	}
	
	PACKAGE static void initialize()
	{
		/* Setup the packet transports */
		transports = new moab_Transports();
		/*
		 * Add a `udp' transport.
		 * XXX Shouldn't moab be telling us about these things?
		 */
		transports.installInterface(new moab_TransportInterface(
			"udp",
			0,
			new moab_IPAddressSpec("0"),
			0));
	}

	public static int getInterfaceCount()
	{
		return transports.getInterfaceCount();
	}

	public static Vector getInterfaces()
	{
		return transports.getInterfaceNameList();
	}

	public static TransportInterface getInterface(String name)
	{
		return (TransportInterface)transports.findInterface(name);
	}
}
