/*
 * moab_Setup.Java
 *
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_NODEOS_PKG;

import java.util.Vector;

import JVM_SYS_PKG.Team;
import JVM_SYS_PKG.TeamHandle;
import JVM_SYS_PKG.ConfigFile;

import JVM_RESOURCES_PKG.NameSpaceGroup;
import JVM_RESOURCES_PKG.NameSpaceResource;

/**
 * Setup the Moab native NodeOS.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
PACKAGE class moab_Setup
{
	PACKAGE static Team nodeosTeam = Team.current();
	
	private static NameSpaceGroup[] kernelNameSpaces;
	
	PACKAGE static NameSpaceGroup[] getImportedNameSpaces()
	{
		return moab_Setup.kernelNameSpaces;
	}

	/**
	 * Initialize the Moab native layer.
	 */
	PACKAGE static void initialize()
	{
		TeamHandle kernelTeam = nodeosTeam.getKernelTeam();
		Vector nameSpaces = new Vector();
		NameSpaceGroup nsg[];
		String ns;
		int lpc;

		System.loadLibrary("kaffenodeos");
		
		nameSpaces.addElement("jnodeos");
		try
		{
			NameSpaceResource.exportClassGroup("jnodeos");
		}
		catch(Exception e)
		{
			Node.printStackTrace(e);
			Node.fatalError("Couldn't export jnodeos class group");
		}

		nsg = new NameSpaceGroup[nameSpaces.size() + 1];
		for( lpc = 0; lpc < nameSpaces.size(); lpc++ )
		{
			/*
			 * XXX We happen to know that the team we give to the
			 * spec will resolve to an internal identifier and
			 * not the actual java object.  So we don't have to
			 * worry about the object going away or not being
			 * relevant in another team.
			 */
			nsg[lpc] = new NameSpaceGroup(
				nodeosTeam,
				(String)nameSpaces.elementAt(lpc),
				"identity");
		}
		/* Attach the kernel name space. */
		nsg[lpc] = new NameSpaceGroup(kernelTeam,
					      "kernel",
					      "identity");
		moab_Setup.kernelNameSpaces = nsg;
		moab_Info.initialize();

		// new java.lang.Thread(new moab_DebugConsole()).start();
	}
}
