/*
 * moab_Transports.Java
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_NODEOS_PKG;

import java.util.Hashtable;

import java.util.Vector;

/**
 * A moab_Transports object tracks all available packet transports.
 *
 * NOTE:  When we refer to transport we're also talking about upper level
 * protocols (e.g. udp) and not just physical network interfaces.
 */
PACKAGE class moab_Transports
{
	/**
	 * Maps protocol names to their moab_TransportInterface objects.
	 */
	private Hashtable interfaces = new Hashtable();

	/**
	 * List of transport interface names.
	 */
	private Vector interfaceNameList = new Vector();

	private int interfaceCount;

	PACKAGE int getInterfaceCount()
	{
		return this.interfaceCount;
	}

	/**
	 * @return The list of transport interface names.
	 */
	PACKAGE Vector getInterfaceNameList()
	{
		return this.interfaceNameList;
	}
	
	/**
	 * @param ti The transport to install.
	 */
	PACKAGE void installInterface(moab_TransportInterface ti)
	{
		// Node.println("Installing interface " + ti.getName());
		this.interfaces.put(ti.getName(), ti);
	}

	/**
	 * @param name The transport name to find.
	 * @return The moab_TransportInterface that corresponds to the given
	 * @return name.
	 */
	PACKAGE moab_TransportInterface findInterface(String name)
	{
		return (moab_TransportInterface)this.interfaces.get(name);
	}

	/**
	 * Query moab about any physical network interfaces.
	 *
	 * @return The number of physical network interfaces.
	 */
	private native int queryInterfaces();

	/**
	 * @param index The index of physical network interface to get.
	 * @return The object that encodes information about the interface.
	 */
	private native moab_TransportInterface getInterfaceAt(int index);

	/**
	 * Query moab for physical transports install them in the new object.
	 */
	PACKAGE moab_Transports()
	{
		moab_TransportInterface ti;
		int lpc, count;

		/* Install any physical network interfaces */
		count = this.queryInterfaces();
		for( lpc = 0; lpc < count; lpc++ )
		{
			ti = this.getInterfaceAt(lpc);
			this.installInterface(ti);
			this.installInterface(
				new moab_TransportInterface("if"
							    + ti.getID(),
							    ti));
		}
		this.interfaceCount = count;
	}

	public String toString()
	{
		String retval = "moab_Transports[interfaces=(";
		int lpc;

		for( lpc = 0; lpc < this.interfaceNameList.size(); lpc++ )
		{
			retval += " "
				+ this.interfaceNameList.elementAt(lpc);
		}
		retval += " )]";
		return retval;
	}
}
