/*
 * pj_CutThroughChannel.Java
 *
 * Copyright (c) 1999, 2000, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_NODEOS_PKG;

import java.io.IOException;

/**
 * Pure Java implementation of a CutThroughChannel.
 * 
 * @see pj_InChannel
 * @see pj_OutChannel
 * @see CutThroughChannel
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
PACKAGE class pj_CutThroughChannel
{
	/**
	 * The OutChan this CutThrough forwards to.
	 */
	private final pj_OutChannel outTo;

	/**
	 * The InChan this CutThrough receives from.
	 */
	private final pj_InChannel inFrom;

	PACKAGE pj_CutThroughChannel(pj_Domain domain,
				     pj_DemultiplexKey classifier,
				     int threadCount,
				     NetworkSpec ons,
				     String inProtocolSpec,
				     String outProtocolSpec,
				     String inAddressSpec,
				     String outAddressSpec,
				     Address destAddr)
		throws IOException,
		PacketAddressFormatException,
		ProtocolSpecFormatException
	{
		// Caller did sanity checks on arguments

		outTo = new pj_OutChannel(domain, ons, outProtocolSpec, outAddressSpec);
		// above might throw IOException

		// XXX can make this a method on pj_CutThroughChannel now.
		DeliverFunc f = new DeliverFunc() {
			public void receive(BufferHandle packet, int payloadStart)
				throws IOException
				{
					outTo.send(packet);
				}
		};

		inFrom = new pj_InChannel(domain,
					  threadCount,
					  classifier,
					  inProtocolSpec,
					  inAddressSpec,
					  f);
		
		domain.addChannel(this);
	}
	
	/**
	 * Add a buffer to the channel
	 */
	PACKAGE void native_addBuffer(BufferHandle pb)
	{
		this.inFrom.native_addBuffer(pb);
	}

	/**
	 * Remove a buffer from the channel
	 */
	PACKAGE BufferHandle native_remBuffer()
	{
		return this.inFrom.native_remBuffer();
	}
	
	PACKAGE void destroy()
	{
		TRACE(Trace.NODEOS_DESTROY, "Destroying " + this);
	}
}


