/*
 * pj_DomainHandle.Java
 *
 * Copyright (c) 2000, 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_NODEOS_PKG;

/**
 * Pure-java backend-implementation of a DomainHandle.
 *
 * XXX since the JVM_SYS_PKG is emulated in pj, we could make the
 * moab_DomainHandle implementation the top-level implementation,
 * and get rid of this class?
 */
PACKAGE class pj_DomainHandle
{
	/**
	 * Pointer to the actual domain.
	 */
	private final Domain domain;

	PACKAGE pj_DomainHandle(Domain f)
	{
		this.domain = f;
	}

	PACKAGE void getStats(pj_DomainStats fs)
	{
		// XXX not in pj
	}

	PACKAGE boolean getInChannelStats(int index, pj_ChanStats cs)
	{
		// XXX no stats yet
		return false;
	}

	PACKAGE boolean getOutChannelStats(int index, pj_ChanStats cs)
	{
		// XXX no stats yet
		return false;
	}


	/**
	 * This method is provided by Importable for moab_DomainHandle...
	 */
	PACKAGE boolean pointsTo(Domain f)
	{
		return this.domain == f;
	}

	PACKAGE void nativeDestroy()
	{
		require(this.domain);
		this.domain.destroy();
	}

	PACKAGE String getName()
	{
#ifdef DEBUG
		return this.domain.getName();
#else
		return null;
#endif
	}

	/**
	 * XXX this method only exists in the pj_ implementation.
	 * Isn't needed in the moab_ version (only used for setting
	 * root domain's name).  Also, would be ugly in Moab version.)
	 */
	PACKAGE void setName(String name)
	{
#ifdef DEBUG
		this.domain.setName(name);
#endif
	}
}
