/*
 * pj_InChannelHandle.Java
 *
 * Copyright (c) 1999, 2000, 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_NODEOS_PKG;

/**
 * A cross-process handle on an pj_InChannel.  Packets can be inserted
 * into an pj_InChannel via a pj_InChannelHandle.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 * @see InChannelHandle
 * @see InChannel
 * @see pj_InChannel
 */
PACKAGE class pj_InChannelHandle
{
	/**
	 * This would be final, except that we need the InChannelHandle
	 * to be created first, and then initialized out of a CommSpace.
	 */
	private pj_InChannel inChan;

	PACKAGE pj_InChannelHandle()
	{
		setChannel(null);
	}

	PACKAGE void setChannel(pj_InChannel inChan)
	{
		// XXX reference counting on the target inChan?
		// XXX cross-process reference!
		this.inChan = inChan;
	}

	PACKAGE void copy(pj_InChannelHandle other)
	{
		setChannel(other.inChan);
	}

	/**
	 * Push a buffer onto the associated InChannel.
	 * <p>
	 * XXX currently deals with fragmented buffers internally.
	 * Should have a 'pushv' function that deals with buffer
	 * aggregates.  That our we should require that aggregates
	 * are linearized by the caller.
	 * <p>
	 * XXX payload start is gross a hack.  Might not even be necessary?
	 */
	PACKAGE boolean handoff(BufferHandle pkt)
	{
		boolean retval = true;

		TRACE(Trace.NODEOS_OUTGOING, "pj_InChannelHandle.push: " +Debug.dbgPacketSummary(pkt, 128));

		require(this.inChan);

		/*
		 * Push into a new BufferHandle in target process's
		 * context.
		 */
		retval = this.inChan.handoff(pkt);
		return retval;
	}

	PACKAGE boolean handoffv(BufferHandle[] bhl)
	{
		NOTIMPLEMENTED("pj_InChannelHandle.pushv()!");
	}

	public String toString()
	{
		return "pj_InChannelHandle[inChan=" +inChan+ "]";
	}
}

