/*
 * pj_StdinStreamLayer.Java
 *
 * Copyright (c) 2000, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_NODEOS_PKG;

import java.io.IOException;

/**
 * pj_StdinStreamLayer is a protocol layer implementation that parses
 * out ANEP packets that are coming over Stdin.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
PACKAGE class pj_StdinStreamLayer
	extends pj_ANEPStreamLayer
{
	private boolean active = false;
	
	PACKAGE pj_StdinStreamLayer()
	{
	}

	/**
	 * There's no real address to apply a mask too so we just accept
	 * everything.
	 */
	private static class DummyAddressMask
		extends pj_ProtocolLayer.PacketAddressMask
	{
		public void parseAddressMask(String addr)
		{
		}

		public int hashCode()
		{
			return 0;
		}

		public boolean equals(Object o)
		{
			boolean retval = false;
			
			if ( o instanceof DummyAddressMask )
				retval = true;
			return retval;
		}
	}

	/**
	 * A StreamReader subclass that uses Stdin as the InputStream.
	 */
	private static class StdinStreamReader
		extends pj_ANEPStreamLayer.StreamReader
	{
		private PacketHandler ph;

		PACKAGE StdinStreamReader(PacketHandler ph)
		{
			// XXX wrong layer to create BufferHandle()
			super(System.in, BufferHandle.create()); 

			this.ph = ph;
		}

		/**
		 * Just pass it off to the packet handler.
		 */
		protected boolean handlePacket(BufferHandle bh)
		{
			return this.ph.handle(bh);
		}
	}

	PACKAGE String getID()
	{
		return "stdio:anep";
	}

	/**
	 * You can't send with this protocol layer so, just panic for now.
	 */
	PACKAGE AddressSpec createAddress(String addr)
		throws PacketAddressFormatException
	{
		panic("Stdin is a receive only layer!");
	}

	PACKAGE PacketAddressMask createAddressMask()
	{
		return new DummyAddressMask();
	}

	PACKAGE void attachHandler(PacketHandler ph)
	{
		/*
		 * We only need a single reader thread so just attach it once
		 */
		synchronized (this)
		{
			if ( !active )
			{
				java.lang.Thread thread;
				StreamReader sr;
				
				sr = new StdinStreamReader(ph);
				thread = new java.lang.Thread(sr);
				thread.setDaemon(true);
				thread.start();
				active = true;
			}
		}
	}

	PACKAGE void detachHandler(PacketHandler ph)
	{
	}

	/**
	 * You can't send with this protocol layer so, just panic for now.
	 */
	PACKAGE void attachGenerator(PacketGenerator pg)
	{
		panic("Stdin is a receive only layer!");
	}

	/**
	 * Synchronous send a packet, blocking until the send
	 * completes.
	 */
	PACKAGE void synchronousSend(PacketGenerator pg, BufferHandle bh)
		throws IOException
	{
		panic("Stdin is a receive only layer!");
	}

	/**
	 * You can't send with this protocol layer so, just panic for now.
	 */
	PACKAGE void generatorReady(PacketGenerator pg)
	{
		panic("Stdin is a receive only layer!");
	}
	
	/**
	 * You can't send with this protocol layer so, just panic for now.
	 */
	PACKAGE void detachGenerator(PacketGenerator pg)
	{
		panic("Stdin is a receive only layer!");
	}
}
