/*
 * pj_ThreadPoolSpec.Java
 *
 * Copyright (c) 1999, 2000, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_NODEOS_PKG;

/**
 * NodeOS ThreadPoolSpec implementation in Java
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
PACKAGE class pj_ThreadPoolSpec
	implements Cloneable
{
	private final int maxThreads;
	private final int stackSize;
	private final ThreadScheduler sched;
	private final int heh;

	public pj_ThreadPoolSpec(int maxThreads,
				 int stackSize,
				 int rate,
				 ThreadScheduler sched)
	{
		require(maxThreads >= 1);
		require(stackSize >= 0);
		require(sched);

		this.maxThreads = maxThreads;
		this.stackSize = stackSize;
		this.sched = sched;
		this.heh = rate;
	}

	PACKAGE int maxThreads()
	{
		return this.maxThreads;
	}

	PACKAGE static int getMaxRate()
	{
		return 100000; // XXX
	}

	public Object clone()
	{
		try
		{
			return super.clone();
		}
		catch(CloneNotSupportedException e)
		{
			throw new InternalError();
		}
	}
}
