/*
 * ICMPTimeExceeded.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_PROTOCOLS_PKG.icmp;

import JANOS_NODEOS_PKG.BufferHandle;

import JANOS_PROTOCOLS_PKG.ip.IPHeader;

import JANOS_UTIL_PKG.Packet;
import JANOS_UTIL_PKG.PacketSegment;

/**
 * An ICMPTimeExceeded is a full ICMP packet used to indicate that some
 * IP packet has run out of TTL.
 */
public class ICMPTimeExceeded
	extends Packet
{
	/**
	 * Construct an ICMPTimeExceeded packet that reports the dropping of
	 * the given packet.
	 *
	 * @param src The source address for this packet.
	 * @param dst The destination address for this packet.
	 * @param bh The buffer containing the packet that ran out of TTL.
	 * @param offset The offset into the buffer where the IP header starts.
	 * @param iplen The length of the IP header in the given buffer.
	 */
	public ICMPTimeExceeded(int src, int dst,
				BufferHandle bh, int offset, int iplen)
	{
		super(new PacketSegment[] {
			new IPHeader((byte)0,
				     (short)0,
				     (short)0,
				     IPHeader.MAX_TTL,
				     IPHeader.PROTO_ICMP,
				     src,
				     dst),
			new ICMPHeader(ICMPHeader.ECHO, (byte)0),
			new ICMPSummarySegment(bh, offset, iplen)
		});
	}
}
