/*
 * RouteTable.Java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_ROUTE_PKG;

/**
 * A read-only wrapper object for the back end.
 */
public class RouteTable
{
	/**
	 * Reference to the actual route table.
	 */
	protected RouteTableBackEnd be;

	/**
	 * Construct a RouteTable object that refers to the given table.
	 *
	 * @param be Reference to the actual route table.
	 */
	public RouteTable(RouteTableBackEnd be)
	{
		require(be);
		
		this.be = be;
	}

	/**
	 * @param address The destination address to find.
	 * @return The RouteEntry matching the given address or null.
	 */
	public synchronized RouteEntry getRoute(int address)
	{
		require(address != 0);
		
		return this.be.getRoute(address);
	}

	/**
	 * @return The number of RouteEntry's in the table.
	 */
	public synchronized int getRouteCount()
	{
		return this.be.getRouteCount();
	}

	/**
	 * @param res An array to fill with RouteEntry's.
	 * @return The number of elements used in the `res' array.
	 */
	public synchronized int getEntries(RouteEntry res[])
	{
		require(res);
		
		return this.be.getEntries(res);
	}

	public synchronized RouteEntry[] getRoutes()
	{
	    RouteEntry retval[] = new RouteEntry[this.getRouteCount()];

	    this.getEntries(retval);
	    return retval;
	}
	
	public String toString()
	{
		return "RouteTable[be="
			+ this.be
			+ "]";
	}
}
