/*
 * RouteTableBackEnd.Java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_ROUTE_PKG;

import java.util.Vector;

/**
 * The main routing table object.
 */
public final class RouteTableBackEnd
{
	/**
	 * The root of the RouteEntry tree.
	 */
	private final RouteTrie root = new RouteTrie(0, 0, null);

	/**
	 * The number of routes in the tree.
	 */
	private int routeCount = 0;

	/**
	 * Construct an empty routing table.
	 */
	public RouteTableBackEnd()
	{
	}

	/**
	 * @param re The entry to add to the table.
	 */
	synchronized void addRoute(RouteEntry re)
	{
		this.routeCount += this.root.addEntry(re);
	}

	/**
	 * @param address The address of the entries to remove.
	 */
	synchronized void removeRoute(int address)
	{
		if( this.root.removeEntry(address) )
			this.routeCount -= 1;
	}

	/**
	 * @param hop The hop to remove from the table.
	 */
	synchronized void removeHop(Object hop)
	{
		this.routeCount -= this.root.removeHop(hop);
	}

	/**
	 * @param address The destination address to find.
	 * @return The RouteEntry matching the given address or null.
	 */
	synchronized RouteEntry getRoute(int address)
	{
		return this.root.lookup(address);
	}

	/**
	 * @return The number of RouteEntry's in the table.
	 */
	synchronized int getRouteCount()
	{
		return this.routeCount;
	}

	/**
	 * @param res An array to fill with the table's RouteEntry's.
	 * @return The number of elements used in the `res' array.
	 */
	synchronized int getEntries(RouteEntry res[])
	{
		return this.root.getEntries(res, 0);
	}

	public String toString()
	{
		return "RouteTableBackEnd[root="
			+ this.root
			+ "; routeCount="
			+ this.routeCount
			+ "]";
	}
}
