/*
 * WriteableRouteTable.Java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_ROUTE_PKG;

/**
 * A read/write wrapper object for the back end.
 */
public class WriteableRouteTable
	extends RouteTable
{
	/**
	 * Construct a WriteableRouteTable object that refers to the given
	 * table.
	 *
	 * @param be Reference to the actual route table.
	 */
	public WriteableRouteTable(RouteTableBackEnd be)
	{
		super(be);
	}

	/**
	 * @param re The entry to add to the table.
	 */
	public synchronized void addRoute(RouteEntry re)
	{
		this.be.addRoute(re);
	}

	/**
	 * @param address The address of the entries to remove.
	 */
	public synchronized void removeRoute(int address)
	{
		this.be.removeRoute(address);
	}

	/**
	 * @param hop The hop to remove from the table.
	 */
	public synchronized void removeHop(Object hop)
	{
		this.be.removeHop(hop);
	}

	public String toString()
	{
		return "WriteableRouteTable["
			+ super.toString()
			+ "]";
	}
}
