/*
 * UdpTest.Java
 *
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package JANOS_TESTS_PKG;

import JANOS_NODEOS_PKG.DomainRun;
import JANOS_NODEOS_PKG.Domain;
import JANOS_NODEOS_PKG.MemSpec;
import JANOS_NODEOS_PKG.Node;
import JANOS_NODEOS_PKG.OutChannel;
import JANOS_NODEOS_PKG.InChannel;
import JANOS_NODEOS_PKG.DeliverFunc;
import JANOS_NODEOS_PKG.DemultiplexKey;
import JANOS_NODEOS_PKG.BufferHandle;
import JANOS_NODEOS_PKG.Address;
import JANOS_NODEOS_PKG.AddressFactory;
import JANOS_NODEOS_PKG.NoSuchNodeAddress;

import JANOS_UTIL_PKG.DomainBase;
import JANOS_UTIL_PKG.SimpleDemultiplexKey;

import java.io.IOException;

public class MulticastUdpTest
	extends DomainBase
{
	private static final boolean VERBOSE = true;

	private static final int RECV_PORT = 33000;
	private static final String MULTICAST_ADDRESS = "239.192.0.50";
	private static final int PACKET_SEND_COUNT = 10;
	private static final int PACKET_RECV_COUNT =  8;
	private static final byte key[] = { 1, 3, 3, 7 };
	
	private InChannel ic;
	private OutChannel oc;
	
	public MulticastUdpTest()
	{
	}

	public void start()
	{
		Node.println("Started MulticastUdpTest");

		try
		{
			start1();
		}
		catch(Throwable th)
		{
			th.printStackTrace();
			throw new Error("Test failed");
		}
	}

	private class MulticastReceiver
		implements DeliverFunc
	{
		private InChannel ic;
		private int received;

		MulticastReceiver()
		{
		}

		public void setInChannel(InChannel ic)
		{
			this.ic = ic;
			this.ic.addBuffers(10);
		}
		
		public void receive(BufferHandle packet, int offset)
		{
			this.received += 1;
			Node.println("Received packet: "
				     + packet.getInt(key.length));
			if( this.received >= PACKET_RECV_COUNT )
				MulticastUdpTest.this.domain.destroy();
			this.ic.addBuffer(packet);
		}

		public String toString()
		{
			return "MulticastReceiver[ic="
				+ this.ic
				+ "; received="
				+ this.received
				+ "]";
		}
	}

	public void start1()
		throws Exception
	{
		MulticastReceiver mr;
		BufferHandle bh;
		int lpc;
		
		this.oc = OutChannel.create(this.domain,
					    "if/ipv4/udp",
					    "*:"
					    + RECV_PORT
					    + ":"
					    + MULTICAST_ADDRESS);
		mr = new MulticastReceiver();
		this.ic = InChannel.create(this.domain,
					   new SimpleDemultiplexKey(this.domain,
								    0,
								    key),
					   "if/ipv4/udp",
					   "*:"
					   + RECV_PORT
					   + ":"
					   + MULTICAST_ADDRESS,
					   mr);
		mr.setInChannel(this.ic);

		bh = BufferHandle.create();
		bh.setLength(128);
		for( lpc = 0; lpc < PACKET_SEND_COUNT; lpc++ )
		{
			bh.setBytes(0, key, 0, key.length);
			bh.setInt(key.length, lpc + 1);
			this.oc.send(bh);
		}
	}

	public String toString()
	{
		return "MulticastUdpTest["
			+ super.toString()
			+ "]";
	}
}

