/*
 * TestMain.Java
 *
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_TESTS_PKG;

import JANOS_NODEOS_PKG.Domain;
import JANOS_NODEOS_PKG.Main;
import JANOS_NODEOS_PKG.Node;
import JANOS_NODEOS_PKG.Address;
import JANOS_NODEOS_PKG.AddressFactory;
import JVM_COMMSPACE_PKG.CommSpaceHandle;
import JVM_COMMSPACE_PKG.CommSpaceElement;

/**
 * A class to provide a main() for testing.  It will startup up a TestDomain
 * and pass it it's command line arguments.
 */
public class TestMain
{
	/**
	 * The address factory from nodeos, this can only be gotten once
	 * so we have to grab it and export it for everyone else
	 */
	private static AddressFactory addrFactory;

	/**
	 * Encoding of the build environment
	 */
	private static final int environment =
#ifdef DEBUG
		Tester.ENV_DEBUG |
#endif
#ifdef NDEBUG
		Tester.ENV_NDEBUG |
#endif
#ifdef EXPOSED
		Tester.ENV_EXPOSED |
#endif
		0;

	/**
	 * Startup the domain and setup any other references
	 */
	private static void setupObjects(String mainArgs[])
	{
		String args[] = new String[mainArgs.length + 1];

		args[0] = JANOS_TESTS_PKG_S + ".TestDomain";
		System.arraycopy(mainArgs, 0, args, 1, mainArgs.length);
		Main.main(args);
		TestMain.addrFactory = new AddressFactory();
	}

	/**
	 * Return the address factory
	 */
	public static AddressFactory getAddressFactory()
	{
		return TestMain.addrFactory;
	}

	/**
	 * Get the build environment encoding
	 */
	public static int getEnvironment()
	{
		return TestMain.environment;
	}

	/**
	 * Print the usage for this classes' main()
	 */
	private static void usage()
	{
		Node.println("usage: "
			     + Tester.class.getName()
			     + " <test-classes>");
	}

	public static void main(String args[]) throws Exception
	{
		Tester.verbose = true;
		if( args.length > 0 )
		{
			TestMain.setupObjects(args);
		}
		else
		{
			TestMain.usage();
			System.exit(1);
		}
	}

}
