/*
 * CPUHogDomain.Java
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_TESTS_PKG.demo;

import JANOS_NODEOS_PKG.Node;
import JANOS_NODEOS_PKG.Domain;
import JANOS_NODEOS_PKG.MemSpec;
import JANOS_NODEOS_PKG.ThreadPoolSpec;
import JVM_COMMSPACE_PKG.CommSpaceElement;

/**
 */
class CPUHogDomain
	extends SubDomain
{
	private int THREAD_COUNT = 20;
	private int SLEEP_TIME = 1;
	private int RATE_DIVISOR = 20;
	
	private boolean dying = false;
	private boolean suck = false;
	
	CPUHogDomain()
	{
		super(null);
		
		try
		{
			this.configure();
		}
		catch(Exception e)
		{
			Node.println("CPUHog configuration failed: " + e);
			Node.printStackTrace(e);
		}
	}

	private void configure()
		throws Exception
	{
		String str;
		
		str = Domain.lookupConfig("CPUHogConfig", "thread-count");
		if( str != null )
			THREAD_COUNT = Integer.parseInt(str);
		str = Domain.lookupConfig("CPUHogConfig", "sleep-time");
		if( str != null )
			SLEEP_TIME = Integer.parseInt(str);
		str = Domain.lookupConfig("CPUHogConfig", "rate-divisor");
		if( str != null )
			RATE_DIVISOR = Integer.parseInt(str);
	}

	private void printConfiguration()
	{
		Node.println("CPUHogDomain configuration:");
		Node.println("  Threads\t" + THREAD_COUNT);
		Node.println("  Sleep\t\t" + SLEEP_TIME);
		Node.println("  Rate divisor\t" + RATE_DIVISOR);
	}

	private class Sucker
		implements Runnable
	{
		public void run()
		{
			while( !CPUHogDomain.this.dying )
			{
				if( !CPUHogDomain.this.suck )
				{
					try
					{
						Thread.sleep(SLEEP_TIME
							     * 1000);
					}
					catch(InterruptedException ie)
					{
					}
				}
			}
		}
	}

	private void spawnWorkerBees()
	{
		int lpc;

		for( lpc = 0; lpc < THREAD_COUNT; lpc++ )
		{
			this.domain.threadStart(new Sucker(), "Sucker " + lpc);
		}
	}
	
	void doRun(CommSpaceElement[] args)
	{
		try
		{
			this.configure();
			this.printConfiguration();
			this.spawnWorkerBees();
		}
		catch(Exception e)
		{
			Node.println("CPUHog configuration failed: " + e);
			Node.printStackTrace(e);
		}
	}

	synchronized void doStartMisbehaving()
	{
		Node.println("CPUHog being bad.");
		this.suck = true;
		return;
	}
	
	void doStopMisbehaving()
	{
		this.suck = false;
		Node.println("CPUHog not being bad.");
		return;
	}

	PACKAGE void kick(Domain pdomain)
	{
		int memSize = THREAD_COUNT * (32 * 1024) + (4 * 1024 * 1024);
		int rate = ThreadPoolSpec.getMaxRate() / RATE_DIVISOR;
		
		super.kick(pdomain,
			   new ThreadPoolSpec(THREAD_COUNT + 1,
					      20 * 1024,
					      rate,
					      null),
			   new MemSpec(memSize));
	}
}
