/*
 * Main.Java
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_TESTS_PKG.demo;

import JANOS_NODEOS_PKG.Domain;
import JANOS_NODEOS_PKG.Node;
import JANOS_NODEOS_PKG.DomainRun;
import JANOS_NODEOS_PKG.MemSpec;
import JANOS_NODEOS_PKG.Address;
import JANOS_NODEOS_PKG.InChannel;
import JANOS_NODEOS_PKG.DomainStats;
import JANOS_NODEOS_PKG.ChanStats;
import JANOS_NODEOS_PKG.Credentials;
import JANOS_NODEOS_PKG.DeliverFunc;
import JANOS_NODEOS_PKG.BufferHandle;
import JANOS_NODEOS_PKG.DomainStartSpec;
import JANOS_NODEOS_PKG.DemultiplexKey;
import JANOS_NODEOS_PKG.ThreadPoolSpec;
import JVM_COMMSPACE_PKG.CommSpaceHandle;
import JVM_COMMSPACE_PKG.CommSpaceElement;
import JANOS_NODEOS_PKG.ProtocolSpecFormatException;
import JANOS_NODEOS_PKG.PacketAddressFormatException;

import JANOS_SERVICES_PKG.arp.ARPHandler;

import JANOS_PROTOCOLS_PKG.udp.UDPHeader;

import java.io.IOException;

/**
 * Main entrypoint for the Janos Java NodeOS DARPA Demo.
 */
public class Main
	extends DomainRun
	implements DeliverFunc
{
	private Domain domain;

	PACKAGE ForwardDomain forwarder;
	PACKAGE MemHogDomain memHog;
	PACKAGE CPUHogDomain cpuHog;
	PACKAGE NetHogDomain netHog;

	/**
	 * A DebugDomain will install a null demux key on a specific
	 * interface and dump any packets it receives on that interface.
	 * Useful if your domain doesn't seem to be receiving the packets
	 * its supposed to received.
	 */
	private final DebugDomain[] debugDomains = new DebugDomain[5];

	private int controlInterface;

	/**
	 * The control interface command input channel.
	 */
	private InChannel chan;

	private final DomainStats fs = new DomainStats();
	private final ChanStats cs = new ChanStats();

	private void printStats()
	{
		if( forwarder != null )
			forwarder.printStats();
		if( memHog != null )
			memHog.printStats();
		if( cpuHog != null )
			cpuHog.printStats();
		if( netHog != null )
			netHog.printStats();

		// Plus this domain:
		this.domain.getStats(fs);
		this.chan.getStats(cs);
		
		Node.println("Domain `Main/Controller' - " +fs);
		Node.println("  InChannel 0: " +cs);
	}

	public Main()
	{
	}

	/**
	 * Entrypoint when started in a domain.
	 */
	protected void run(Domain domain,
			   String args[],
			   CommSpaceHandle commSpaceHandle)
	{
		this.domain = domain;

		try
		{
			Stats st;
			
			configure(args);
			startDomains();
#if 0
			new Thread(new Runnable() {
				public void run() {
					Node.println("Stat thread starting");
					while( true )
					{
						try
						{
							Thread.sleep(1000);
						}
						catch(InterruptedException e)
						{
						}
						Main.this.printStats();
					}
				}
			}).start();
#endif
			st = new Stats(this.domain, this);
			st.configure();
			new Thread(st).start();
		}
		catch (Exception e)
		{
			Node.println("Exception: " + e);
			Node.printStackTrace(e);
			FATALEXCEPTION(e, "bah");
		}
	}

	private void configure(String[] args)
	{
		if (true)
			this.forwarder = new ForwardDomain();
		else
			this.forwarder = null;

		if (true)
			this.netHog = new NetHogDomain();
		else
			this.netHog = null;

		if (true)
			this.memHog = new MemHogDomain();
		else
			this.memHog = null;

		if (true)
			this.cpuHog = new CPUHogDomain();
		else
			this.cpuHog = null;

		String str = Domain.lookupConfig("ControlConfig", "control-interface");
		int control = -1;
		
		if( str != null )
		{
			try
			{
				control = Integer.parseInt(str);
			}
			catch(NumberFormatException e)
			{
			}
		}
		this.controlInterface = control;
		byte ipaddr[] = { 0, 0, 0, 0 };
		str = Domain.lookupConfig("ControlConfig", "address");
		try
		{
			int ip = Address.fromString(str);
			
			ipaddr[0] = (byte)((ip >> 24) & 0xFF);
			ipaddr[1] = (byte)((ip >> 16) & 0xFF);
			ipaddr[2] = (byte)((ip >>  8) & 0xFF);
			ipaddr[3] = (byte)((ip      ) & 0xFF);
			ARPHandler.config_iface(control, ipaddr);
		}
		catch(Exception e)
		{
		}

		int dbgf = 0;
		if (false)
			this.debugDomains[dbgf++] = new DebugDomain(0);
		if (false)
			this.debugDomains[dbgf++] = new DebugDomain(1);
		if (false)
			this.debugDomains[dbgf++] = new DebugDomain(2);
		if (false)
			this.debugDomains[dbgf++] = new DebugDomain(this.controlInterface);

		// startTerminationTimeout(60);
	}

	private static void startTerminationTimeout(final int sec)
	{
		// XXX no event support yet...
		
		Thread t = new Thread() {
			public void run() {
				try 
				{
					Thread.sleep(sec * 1000);
				}
				catch (InterruptedException ie)
				{
					Node.println("termination thread interrupted.");
				}
				Node.println("Time's up.  Shutting down.");
				Node.shutdown();
			}
		};
		t.start();
	}

	private void startDomains()
		throws ProtocolSpecFormatException, PacketAddressFormatException, IOException
	{

		if (this.forwarder != null)
			this.forwarder.kick(this.domain);

		if (this.netHog != null)
			this.netHog.kick(this.domain);

		if (this.memHog != null)
			this.memHog.kick(this.domain);
		
		if (this.cpuHog != null)
			this.cpuHog.kick(this.domain);

		for (int i = 0; i < debugDomains.length; i++)
			if (debugDomains[i] != null)
				debugDomains[i].kick(this.domain);

		startControlChannel();
	}

	private void startControlChannel()
		throws ProtocolSpecFormatException, PacketAddressFormatException, IOException
	{
		DemultiplexKey dk = new DemultiplexKey(this.domain);

		// XXX these don't belong here
		final int   ETHERNET_TYPE_OFFSET = 12;
		final int   ETHERNET_HEADER_LENGTH = 14;
		final short ETHERNET_TYPE_IP = 0x0800;
		final int    IP_PROTO_OFFSET = 9;
		final byte   IP_TYPE_UDP = 17;
		final int    IP_HEADER_LENGTH = 20;
		final int      UDP_PORT_OFFSET = 2;
		final short      CTRLPORT = 0x270f; /* === htons(9999) */
		
		// Don't want to use any sort of built-in IP or UDP
		// support, so we'll build a key that matches most of
		// what we want...

		dk.addETH(0,
			  DemultiplexKey.ETH_ANYSADDR |
			  DemultiplexKey.ETH_ANYDADDR,
			  null,
			  null,
			  ETHERNET_TYPE_IP);

		dk.addIPv4(0,
			   DemultiplexKey.IPV4_ANYSADDR |
			   DemultiplexKey.IPV4_ANYDADDR,
			   0, 0,
			   0, 0,
			   IP_TYPE_UDP);

		// match UDP port number in the UDP header
		dk.add(UDPHeader.OFFSET_DESTINATION_PORT, (short)CTRLPORT);

		Node.println("Control listen config:");
		Node.println("  Control port: " +(((CTRLPORT&0xff00)>>8)|((CTRLPORT&0xff)<<8)));
		Node.println("  DemuxKey: " +dk);

		this.chan = InChannel.create(this.domain,
					     dk,
					     "eth" + this.controlInterface,
					     "0:0",
					     this);
		this.chan.addBuffers(10);
	}

	private void handleControlPacket(int opcode, int subopcode, int[] data)
	{
		final int CTRLPKTOP_FOO = 1;
		final int CTRLPKTOP_NETHOG = 2;
		final int   NETHOGCTRL_ON = 1;
		final int   NETHOGCTRL_OFF = 2;
		final int   NETHOGCTRL_BANDW = 3; /* data[0] is new bandw in ??? */
		final int CTRLPKTOP_CPUHOG = 3;
		final int   CPUHOGCTRL_ON = 1;
		final int   CPUHOGCTRL_OFF = 2;
		final int   CPUHOGCTRL_RATE = 3;
		final int CTRLPKTOP_MEMHOG = 4;	/* mem hog is in java-demo only */
		final int   MEMHOGCTRL_ON = 1;
		final int   MEMHOGCTRL_OFF = 2;
		final int CTRLPKTOP_PQ = 5;
		final int   PQCTRL_ON = 1;
		final int   PQCTRL_OFF = 2;
		final int CTRLPKTOP_CPUSCHED = 6;
		final int   CPUSCHEDCTRL_ON = 1;
		final int   CPUSCHEDCTRL_OFF = 2;
		final int CTRLPKTOP_MEMLIMIT = 7;
		final int   MEMLIMITCTRL_ON = 1;
		final int   MEMLIMITCTRL_OFF = 2;
		final int CTRLPKTOP_STATS = 8;
		final int   STATSCTRL_TOTAL = 1;
		final int   STATSCTRL_DELTA = 2;
		final int CTRLPKTOP_VERBOSE = 9;
		final int   VERBOSECTRL_ON = 1;
		final int   VERBOSECTRL_OFF = 2;
		final int CTRLPKTOP_EXIT = 10;
		
		/* 
		Node.println("Control packet is: op=" +opcode+
			     "; subop=" +subopcode+
			     "; data[0-3]=0x" +
			     Integer.toHexString(data[0])+ ",0x" +
			     Integer.toHexString(data[1])+ ",0x" +
			     Integer.toHexString(data[2])+ ",0x" +
			     Integer.toHexString(data[3]));
		*/

		switch(opcode)
		{
		case CTRLPKTOP_FOO:
			Node.println("Got the test control op: "+
				     "Subop=" +subopcode +"; "+
				     "data[0]="+data[0] +"; "+
				     "data[1]="+data[1]);
			break;
		case CTRLPKTOP_NETHOG:
			switch(subopcode)
			{
			case NETHOGCTRL_ON:
				this.netHog.startMisbehaving();
				break;
			case NETHOGCTRL_OFF:
				this.netHog.stopMisbehaving();
				break;
			case NETHOGCTRL_BANDW:
				Node.println("NetHog control: ignored bw");
				break;
			default:
				Node.println("NetHog Control: "
					     + "Unknown subopcode "
					     + subopcode);
				break;
			}
			break;
		case CTRLPKTOP_CPUHOG:
			switch(subopcode)
			{
			case CPUHOGCTRL_ON:
				this.cpuHog.startMisbehaving();
				break;
			case CPUHOGCTRL_OFF:
				this.cpuHog.stopMisbehaving();
				break;
			default:
				Node.println("CPUHog Control: "
					     + "Unknown subopcode "
					     + subopcode);
				break;
			}
			break;
		case CTRLPKTOP_MEMHOG:
			switch(subopcode)
			{
			case MEMHOGCTRL_ON:
				this.memHog.startMisbehaving();
				break;
			case CPUHOGCTRL_OFF:
				this.memHog.stopMisbehaving();
				break;
			default:
				Node.println("MemHog Control: "
					     + "Unknown subopcode "
					     + subopcode);
				break;
			}
			break;
		case CTRLPKTOP_PQ:
			switch(subopcode)
			{
			case PQCTRL_ON:
				Node.println("PQ Control: on");
				Node.enablePQ();
				break;
			case PQCTRL_OFF:
				Node.println("PQ Control: off");
				Node.disablePQ();
				break;
			default:
				Node.println("PQ Control: "
					     + "Unknown subopcode "
					     + subopcode);
				break;
			}
			break;
		case CTRLPKTOP_CPUSCHED:
			switch(subopcode)
			{
			case CPUSCHEDCTRL_ON:
				Node.println("CPUsched Control: on");
				Node.enableCPUScheduling();
				break;
			case CPUSCHEDCTRL_OFF:
				Node.println("CPUsched Control: off");
				Node.disableCPUScheduling();
				break;
			default:
				Node.println("CPUsched Control: "
					     + "Unknown subopcode "
					     + subopcode);
				break;
			}
			break;
		case CTRLPKTOP_STATS:
			switch(subopcode)
			{
			case STATSCTRL_TOTAL:
				Node.println("Stats control: total");
				this.printStats();
				break;
			case STATSCTRL_DELTA:
				Node.println("Stats control: delta (unimplemented)");
				break;
			default:
				Node.println("Stats Control: "
					     + "Unknown subopcode "
					     + subopcode);
				break;
			}
			break;
		case CTRLPKTOP_VERBOSE:
			switch(subopcode)
			{
			case VERBOSECTRL_ON:
				Node.println("Verbose control: on");
				break;
			case VERBOSECTRL_OFF:
				Node.println("Verbose control: off");
				break;
			default:
				Node.println("Verbose control: "
					     + "Unknown subopcode "
					     + subopcode);
				break;
			}
			break;
		case CTRLPKTOP_EXIT:
			Node.println("Shutdown control: initiating shutdown");
			Node.shutdown();
			break;
		default:
			Node.println("Unknown op:" +opcode);
			break;
		}
	}

	public void receive(BufferHandle packet, int payloadOffset)
		throws IOException
	{
		// Node.println("received control packet");
		// Node.println("offset=" +payloadOffset+ "; len=" +packet.getLength());

		// Extract a "ctrl_packet_t"
		int opcode;
		int subopcode;
		int[] data = new int[10];

		payloadOffset+=4; // skip the rest of the UDP header

		// XXX check IP header checksum?
		// XXX check UDP checksum?

		opcode = packet.getInt(payloadOffset);
		subopcode = packet.getInt(payloadOffset+4);
		packet.getInts(payloadOffset+8, data, 0, data.length);

		// Have copied everything we need from packet, recycle it.
		this.chan.addBuffer(packet);

		// Figure out what the packet means...
		handleControlPacket(opcode, subopcode, data);
	}
}
