/*
 * MemHogDomain.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_TESTS_PKG.demo;

import JANOS_NODEOS_PKG.Node;
import JANOS_NODEOS_PKG.Domain;
import JANOS_NODEOS_PKG.MemSpec;
import JANOS_NODEOS_PKG.ThreadPoolSpec;
import JVM_COMMSPACE_PKG.CommSpaceElement;

/**
 */
class MemHogDomain
	extends SubDomain
{
	private int THREAD_COUNT = 1;
	private int SLEEP_TIME = 1;
	private int RATE_DIVISOR = 20;
	
	private boolean dying = false;
	private boolean chew = false;
	
	MemHogDomain()
	{
		super(null);
		
		try
		{
			this.configure();
		}
		catch(Exception e)
		{
			Node.println("MemHog configuration failed: " + e);
			Node.printStackTrace(e);
		}
	}

	private void configure()
		throws Exception
	{
		String str;
		
		str = Domain.lookupConfig("MemHogConfig", "thread-count");
		if( str != null )
			THREAD_COUNT = Integer.parseInt(str);
		str = Domain.lookupConfig("MemHogConfig", "sleep-time");
		if( str != null )
			SLEEP_TIME = Integer.parseInt(str);
		str = Domain.lookupConfig("MemHogConfig", "rate-divisor");
		if( str != null )
			RATE_DIVISOR = Integer.parseInt(str);
	}

	private void printConfiguration()
	{
		Node.println("MemHogDomain configuration:");
		Node.println("  Threads\t" + THREAD_COUNT);
		Node.println("  Sleep\t\t" + SLEEP_TIME);
		Node.println("  Rate divisor\t" + RATE_DIVISOR);
	}

	private class Chewer
		implements Runnable
	{
		public Chewer()
		{
		}

		public void run()
		{
			/*
			 * Use an array since we can't be sure a stack value
			 * is really nulled out or not.
			 */
			Object root[] = { null };
			Object curr[] = null;
			
			while( !MemHogDomain.this.dying )
			{
				try
				{
					if( MemHogDomain.this.chew )
					{
						if( curr == null )
						{
							curr = new Object[500];
							root[0] = curr;
						}
						else
						{
							curr[0] = new Object[500];
							curr[1] = new Object[500];
							curr = (Object [])curr[1];
						}
						Thread.sleep(10);
					}
					else
					{
						Thread.sleep(SLEEP_TIME * 1000);
					}
				}
				catch(InterruptedException ie)
				{
				}
				catch(OutOfMemoryError e)
				{
					root[0] = null;
					curr = null;
				}
			}
		}
	}

	private void spawnWorkerBees()
	{
		int lpc;

		for( lpc = 0; lpc < THREAD_COUNT; lpc++ )
		{
			this.domain.threadStart(new Chewer(), "Chewer " + lpc);
		}
	}
	
	void doRun(CommSpaceElement[] args)
	{
		try
		{
			this.configure();
			this.printConfiguration();
			this.spawnWorkerBees();
		}
		catch(Exception e)
		{
			Node.println("MemHog configuration failed: " + e);
			Node.printStackTrace(e);
		}
	}
	
	void doStartMisbehaving()
	{
		Node.println("MemHog being bad.");
		this.chew = true;
		return;
	}
	
	void doStopMisbehaving()
	{
		// cleanup mess before retiring
		System.gc();
		System.gc();
		Node.println("MemHog not being bad.");
		this.chew = false;
		return;
	}
	
	PACKAGE void kick(Domain pdomain)
	{
		int memSize = THREAD_COUNT * (20 * 1024) + (2 * 1024 * 1024);
		int rate = ThreadPoolSpec.getMaxRate() / RATE_DIVISOR; // XXX
		
		super.kick(pdomain,
			   new ThreadPoolSpec(THREAD_COUNT + 1,
					      20 * 1024,
					      rate,
					      null),
			   new MemSpec(memSize));
	}
}
