/*
 * AddressTester.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_TESTS_NODEOS_PKG;

import java.net.InetAddress;

import JANOS_TESTS_PKG.*;

import JANOS_NODEOS_PKG.Address;
import JANOS_NODEOS_PKG.AddressFormatException;

public class AddressTester extends Tester
{
	public void testUnit() throws TestFailedException
	{
		Address addr, addrCopy;
		InetAddress iaddr;
		int lpc;

		try
		{
			iaddr = InetAddress.getLocalHost();
		}
		catch(java.net.UnknownHostException e)
		{
			return;
		}
		
		this.enterSection(ENV_NONE, "Valid fromString()/toString()'s");
		try
		{
			String addrStrings[] =
			{
				"0.0.0.0",
				"0.1.2.4",
				"8.16.32.64",
				"255.255.255.255"
			};
			int addrInts[] =
			{
				0x00000000,
				0x00010204,
				0x08102040,
				0xFFFFFFFF
			};

			for( lpc = 0; lpc < addrStrings.length; lpc++ )
			{
				this.test(Address.
					  fromString(addrStrings[lpc])
					  == addrInts[lpc]);
				this.test(Address.toString(addrInts[lpc]).
					  equals(addrStrings[lpc]));
			}
		}
		catch(AddressFormatException e)
		{
			throw new TestFailedException(this);
		}

		this.enterSection(ENV_NONE, "Invalid fromString()'s");
		String badAddrs[] =
		{
			"",
			"256.256.256.256",
			"0",
			"0.0",
			"0.0.0",
			"0.0.0. ",
			"0.0.0.a",
			"This.really.shouldn't.work",
			"Nor this",
			"0 . 0 . 0 . 0",
			"-1.-1.-1.-1"
		};

		for( lpc = 0; lpc < badAddrs.length; lpc++ )
		{
			try
			{
				Address.fromString(badAddrs[lpc]);
				throw new TestFailedException(this);
			}
			catch(AddressFormatException e)
			{
				; /* This is supposed to happen */
			}
		}

		this.enterSection(ENV_EXPOSED, "getAddressFactory()");

#ifdef EXPOSED
		if( Address.globalFactory == null)
			throw new TestFailedException("globalFactory failed to start unset", this);
		
		if( Address.getAddressFactory() == null )
			throw new TestFailedException("getAddressFactory() returned null", this);
#endif
		
		this.enterSection(ENV_EXPOSED, "Constructors/Accessors");
#ifdef EXPOSED
		/* Regular constructor */
		addr = new Address(0x00010204, "if/ipv4/udp", "localhost:1024");
		addrCopy = new Address(addr);
		this.test(addr.getANAddress() == 0x00010204);
		this.test(addr.getANAddress() == addrCopy.getANAddress());
		this.test(addr.getAddressSpec() == addrCopy.getAddressSpec());
		this.test(addr.getProtocolSpec() == addrCopy.getProtocolSpec());
#endif
	}
}
