/*
 * DemultiplexKeyTester.Java
 *
 * Copyright (c) 1999, 2000, 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package JANOS_TESTS_NODEOS_PKG;

import JANOS_TESTS_PKG.*;
import JANOS_NODEOS_PKG.*;

public class DemultiplexKeyTester extends Tester
{
	/**
	 * Generate a random array of bytes for a key
	 *
	 * @param length The length of the array
	 * @return An array of bytes with random element values
	 */
	private byte[] generateKeyBytes(int length)
	{
		byte retval[] = new byte[length];
		int lpc;

		for( lpc = 0; lpc < length; lpc++ )
		{
			retval[lpc] = (byte)(Math.random() * 10);
		}
		return retval;
	}

	/**
	 * Create an array from a section of another
	 *
	 * @param key The array to create the slice from
	 * @param offset The start of the section in key to copy
	 * @param length The length of the section in key to copy
	 * @return A new byte array that has the values from the section
	 * of key specified by the parameters
	 */
	private byte[] generateSlice(byte key[], int offset, int length)
	{
		byte retval[] = new byte[length];

		System.arraycopy(key, offset, retval, 0, length);
		return retval;
	}

	/**
	 * Create a byte array that won't match the specified subsection of key
	 *
	 * @param key The byte array to generate the bad slice from
	 * @param offset The start of the section in key to create the bad
	 * slice from
	 * @param length The length of the section in key
	 * @return A byte array that won't match the section in key specified
	 * by the parameters
	 */
	private byte[] generateBadSlice(byte key[], int offset, int length)
	{
		byte retval[] = new byte[length];
		int lpc;

		for( lpc = 0; lpc < length; lpc++ )
		{
			retval[lpc] = (byte)(key[offset + lpc] + 1);
		}
		return retval;
	}
	
	public void testUnit() throws TestFailedException
	{
		byte keyBytes[], keySlice[], badSlice[];
		int keyLength = 16;
		DemultiplexKey dk;
		int lpc;

		keyBytes = this.generateKeyBytes(keyLength);
		dk = new DemultiplexKey(Domain.currentDomain());
		dk.add(keyLength, keyBytes, null);
		this.enterSection(0, "Accessor");
		this.test(dk.length() == keyLength);
		this.enterSection(0, "Comparisons");
		this.test(dk.matches(keyBytes, 0, keyLength) == keyLength);
		this.test(dk.matches(keyBytes, 0, keyLength - 1) == -1);
		try
		{
			this.test(dk.matches(keyBytes, 1, keyLength) == -1);
			throw new TestFailedException(
				"Bad payload not detected");
		}
		catch(Error e)
		{
			;
		}
		this.test(dk.matches(keyBytes, 1, keyLength - 1) == -1);
		this.test(dk.matches(this.generateBadSlice(keyBytes,
							   0,
							   keyLength),
				     0,
				     keyLength) == -1);
	}
}
