/*
 * DomainClassLoaderTester.Java
 *
 * Copyright (c) 1999, 2000, 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package JANOS_TESTS_NODEOS_PKG;

import JANOS_TESTS_PKG.*;
import JANOS_NODEOS_PKG.*;

public class DomainClassLoaderTester extends Tester
{
	/* This is pretty basic and could stand to be enhanced quite a bit */
	public void testUnit() throws TestFailedException
	{
		this.enterSection(ENV_EXPOSED, "DomainClassLoader");
#ifdef EXPOSED
		DomainClassSpec fcSpec = new DomainClassSpec();
		DomainClassLoader fcl;

		fcSpec.addSystemPackage("java.lang");
		fcSpec.addSystemPackage("java.util");
		fcSpec.sealPackage("java.lang");
		fcSpec.sealPackage("java.util");
		this.enterSection(0, "Constructor");
		fcl = new DomainClassLoader(Domain.currentDomain(), fcSpec);

		this.enterSection(0, "Load local non-system class");
		try
		{
			Class cl = fcl.loadClass(JANOS_TESTS_NODEOS_PKG_S
						 + ".DomainClassLoaderTester",
						 true);
			this.test(cl.getClassLoader() == fcl);
		}
		catch(Exception e)
		{
			throw new TestFailedException(this);
		}
		
		fcl = new DomainClassLoader(Domain.currentDomain(), fcSpec);
		fcl.addSystemPackage(JANOS_TESTS_NODEOS_PKG_S);

		this.enterSection(0, "Load local system class");
		try
		{
			Class cl = fcl.loadClass(JANOS_TESTS_NODEOS_PKG_S
						 + ".DomainClassLoaderTester",
						 true);
			this.test(cl.getClassLoader() != fcl);
		}
		catch(ClassNotFoundException e)
		{
			throw new TestFailedException(this);
		}
		
		this.enterSection(0, "Load bogus system class");
		try
		{
			fcl.loadClass(JANOS_TESTS_NODEOS_PKG_S
				      + ".BadDomainClassLoaderTesterClass",
				      true);
			throw new TestFailedException(this);
		}
		catch(ClassNotFoundException e)
		{
		}
		
		this.enterSection(0, "Access control check");
		try
		{
			fcl.enableAccessControls(true);
			fcl.loadClass(JANOS_NODEOS_PKG_S + ".Address");
			throw new TestFailedException(this);
		}
		catch(ClassNotFoundException e)
		{
		}
		try
		{
			fcl.enableAccessControls(false);
			fcl.loadClass(JANOS_NODEOS_PKG_S + ".Address");
		}
		catch(ClassNotFoundException e)
		{
			throw new TestFailedException(this);
		}
		
		this.enterSection(0, "Package seal check");
		try
		{
			fcl.addClass("java.lang.IAmANaughtyClass",
				     new byte[0]);
			throw new TestFailedException(this);
		}
		catch(SecurityException se)
		{
		}
		try
		{
			fcl.addClass("IAmANaughtyClass", new byte[0]);
			throw new TestFailedException(this);
		}
		catch(SecurityException se)
		{
		}
#endif
	}
}
