/*
 * OutChannelTester.Java
 *
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package JANOS_TESTS_NODEOS_PKG;

import java.net.DatagramSocket;
import java.net.DatagramPacket;
import java.io.IOException;

import JANOS_TESTS_PKG.*;
import JANOS_NODEOS_PKG.*;
import JANOS_UTIL_PKG.SimpleDemultiplexKey;

public class OutChannelTester extends Tester
{
	private final String host = "localhost";
	private int port;
	
	private final int channels = 5;
	private final byte messages[][] = new byte[10][];

	private final int packets = 100;
	private int packetsSent[] = new int[messages.length];
	private int packetsDelivered[] = new int[messages.length];
	private static final byte[] exitPacket = {-0x22, -0x53, -0x42, -0x11};

	/**
	 * Runnable class used to receive packets
	 */
	private class Receiver implements Runnable
	{
		private DatagramSocket ds;
		private OutChannelTester oct;
		private final SimpleDemultiplexKey exitKey =
			new SimpleDemultiplexKey(Domain.currentDomain(),
						 0,
						 exitPacket);

		public Receiver()
			throws IOException
		{
			this.ds = new DatagramSocket();
			OutChannelTester.this.port = this.ds.getLocalPort();
		}

		private boolean handlePacket(DatagramPacket dp)
		{
			boolean retval = false;
			byte data[] = dp.getData();

			if( this.exitKey.matches(data, 0, data.length) >= 0 )
				retval = true;
			return retval;
		}

		public void run()
		{
			try
			{
				final int packetSize = 8192;
				DatagramPacket dp;

				dp = new DatagramPacket(new byte[packetSize],
							packetSize);
				boolean done = false;
				
				while( !done )
				{
					this.ds.receive(dp);
					done = this.handlePacket(dp);
				}
			}
			catch(IOException e)
			{
			}
		}
	}

	/**
	 * Generates the messages to be sent, just a bunch of random bytes
	 */
	private void generateMessages()
	{
		int lpc, lpc2;

		for( lpc = 0; lpc < messages.length; lpc++ )
		{
			messages[lpc] = new byte[lpc + 2];
			for( lpc2 = 0; lpc2 < (lpc + 2); lpc2++ )
			{
				messages[lpc][lpc2] =
					(byte)(Math.random() * 10);
			}
		}
	}

	/**
	 * Send a message over the given out channel
	 */
	private void sendMessage(OutChannel oc, byte message[])
		throws Exception
	{
		BufferHandle bh;

		bh = BufferHandle.create();
		bh.newBuffer(0, message.length);
		bh.setBytes(0, message, 0, message.length);
		oc.send(bh);
	}
	
	public void testUnit() throws TestFailedException
	{
		BufferHandle bh;
		OutChannel oc;
		Address addr;
		int anAddr;
		int lpc;

		this.enterSection(ENV_NONE, "Construction/sending");
		this.generateMessages();
		try
		{
			Domain.currentDomain().
				threadStart(new Receiver(), "Receiver");
			anAddr = Address.fromString("18.1.0.2");
			addr = TestMain.getAddressFactory().
				newAddress(anAddr,
					   "if/ipv4/udp",
					   "*:" + this.port + ":localhost");
			oc = OutChannel.create(Domain.currentDomain(),
					       addr.getProtocolSpec(),
					       addr.getAddressSpec());
			for( lpc = 0; lpc < messages.length; lpc++ )
			{
				this.sendMessage(oc, messages[lpc]);
			}
			this.sendMessage(oc, exitPacket);
		}
		catch(Exception e)
		{
			throw new TestFailedException(this);
		}
	}
}
