/*
 * SemaphoreTester.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package JANOS_TESTS_NODEOS_PKG;

import JANOS_TESTS_PKG.*;
import JANOS_NODEOS_PKG.*;

public class SemaphoreTester extends Tester
{
	private Semaphore sem;
	private int dataCount = -1;

	/**
	 * A simple runnable class that will test signal and block for a
	 * semaphore
	 */
	private class Simple implements Runnable
	{
		public void run()
		{
			for( int i = 0; i < 16; i++ )
			{
				for( int j = 0; j < 16; j++ )
					SemaphoreTester.this.sem.signal();
				for( int j = 0; j < 8; j++ )
					SemaphoreTester.this.sem.block();
			}
			for( int i = 0; i < (16 * 8); i++ )
			{
				SemaphoreTester.this.sem.block();
			}
		}
	}

	/**
	 * Cause the current thread to sleep for some amount of time
	 */
	private void delay()
	{
		try
		{
			java.lang.Thread.sleep(10);
		}
		catch(InterruptedException e)
		{
		}
	}

	/**
	 * A runnable class that acts as a producer of data
	 */
	private class Producer implements Runnable
	{
		private static final int maxDataCount = 64;

		public void run()
		{
			int lpc;

			SemaphoreTester.this.dataCount = this.maxDataCount;
			for( lpc = 0; lpc < this.maxDataCount; lpc++ )
			{
				SemaphoreTester.this.dataCount--;
				SemaphoreTester.this.delay();
				SemaphoreTester.this.sem.signal();
			}
		}
	}

	/**
	 * A runnable class that acts as a consumer of producer's data
	 */
	private class Consumer implements Runnable
	{
		public void run()
		{
			while( SemaphoreTester.this.dataCount != 0 )
			{
				SemaphoreTester.this.sem.block();
			}
		}
	}
	
	public void testUnit() throws TestFailedException
	{
		java.lang.Thread simple, producer, consumer;
		WatchDog watchDog;
		int lpc;

		this.enterSection(ENV_NONE, "Constructor");
		this.sem = new Semaphore();
		simple = new java.lang.Thread(new Simple());
		this.enterSection(ENV_NONE, "signal/block");
		try
		{
			simple.start();
			simple.join(10000);
			if( simple.isAlive() )
				throw new TestFailedException(this);
		}
		catch(InterruptedException e)
		{
			throw new TestFailedException(this);
		}
		watchDog = new WatchDog(10000,
					java.lang.Thread.currentThread());
		producer = new java.lang.Thread(new Producer());
		consumer = new java.lang.Thread(new Consumer());
		try
		{
			consumer.start();
			producer.start();
			consumer.join();
			producer.join();
			watchDog.interrupt();
		}
		catch(InterruptedException e)
		{
			throw new TestFailedException(this);
		}
	}
}
