/*
 * PaddingPacketSegment.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_UTIL_PKG;

import JANOS_NODEOS_PKG.BufferHandle;

/**
 * A PaddingPacketSegment implements the PacketSegment interface for `n'
 * padding bytes.
 */
public class PaddingPacketSegment
	implements PacketSegment
{
	/**
	 * The number of padding bytes that make up this segment.
	 */
	private int length;

	/**
	 * @param length The number of padding bytes in the segment.
	 */
	public PaddingPacketSegment(int length)
	{
		this.length = length;
	}
	
	/**
	 * @return The length of the pad.
	 */
	public int getLength()
	{
		return this.length;
	}

	public int marshal(BufferHandle bh, int offset)
	{
		return offset + this.length;
	}
	
	public int unmarshal(BufferHandle bh, int offset)
	{
		return offset + this.length;
	}

	public String toString()
	{
		return "[PaddingPacketSegment length="
			+ this.length
			+ "]";
	}
}
